Package: statwitness
Type: Package
Title: Model-Aware Validation and Audit Certificates for Statistical
        Analyses
Version: 0.1.0
Authors@R: 
    person(
      given = "Georgios P.",
      family = "Georgiou",
      email = "georgiou.georg@unic.ac.cy",
      role = c("aut", "cre")
    )
Description: Provides model-aware behavioral validation and audit
    certificates for statistical analyses. Controlled transformations and
    model-specific diagnostic checks are organized across five domains:
    computational integrity, numerical stability, design adequacy, assumption
    screening, and influence stability. Supported workflows include linear
    models, generalized linear models, classical and repeated-measures analyses
    of variance, mixed-effects models fitted using 'lme4' or 'glmmTMB', and
    survival models fitted using 'survival'. Checks are selected according to
    registered applicability conditions for each model class. The resulting
    certificates describe computational behavior and selected diagnostic
    findings; they do not establish causal validity, model correctness, or
    scientific appropriateness.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: graphics, reformulas, stats, tools
Suggests: afex, glmmTMB, knitr, lme4, rmarkdown, survival, testthat (>=
        3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/George33cy/statwitness
BugReports: https://github.com/George33cy/statwitness/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-12 10:13:36 UTC; georg
Author: Georgios P. Georgiou [aut, cre]
Maintainer: Georgios P. Georgiou <georgiou.georg@unic.ac.cy>
Repository: CRAN
Date/Publication: 2026-07-21 10:40:08 UTC

export(audit_plan)
importFrom(reformulas,findbars,nobars)
importFrom(stats,setNames)
export(statwitness)
export(statwitness_repeated)
export(witness_registry)
S3method(plot,statwitness_audit)
S3method(print,statwitness_audit)
S3method(print,summary.statwitness_audit)
S3method(summary,statwitness_audit)

# statwitness

<!-- badges: start -->
[![R-CMD-check](https://github.com/George33cy/statwitness/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/George33cy/statwitness/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

`statwitness` provides model-aware behavioral validation and audit certificates for statistical analyses.

The package applies controlled transformations with known expected consequences, refits the analysis, and verifies that the fitted model responds correctly. Examples include reordering observations, changing measurement units, relabeling factors, reconstructing predictions, and refitting models with alternative optimizers.

These behavioral checks are combined with model-specific diagnostics for numerical stability, design adequacy, assumptions, and influential observations.

## Audit domains

`statwitness` organizes checks into five domains.

### Computational integrity

Verifies:

* analysis-sample reconstruction;
* model-matrix and prediction reconstruction;
* row-order invariance;
* factor- and grouping-label invariance;
* unit conversion;
* outcome translation;
* parameterization behavior.

### Numerical stability

Evaluates:

* rank deficiency;
* numerical conditioning;
* convergence;
* singularity;
* boundary estimates;
* optimizer agreement.

### Design adequacy

Screens:

* ANOVA cell balance and empty cells;
* repeated-measures completeness;
* random-effects design;
* potential random-slope requirements;
* survival-event information.

### Assumption screening

Provides model-specific checks for:

* residual shape;
* unequal residual variance;
* GLM dispersion;
* numerical separation;
* proportional hazards.

### Influence stability

Evaluates influential:

* observations;
* participants;
* clusters;
* raters;
* survival-model cases.

The same audit domains are used across model families, but the individual checks are model-specific. A check is run only when its expected behavior is justified for the fitted model.

## Supported analyses

Version 0.1.0 supports:

* linear regression with `stats::lm()`;
* classical and factorial ANOVA represented by `lm()` or `aov()`;
* generalized linear models with `stats::glm()`;
* linear mixed-effects models with `lme4::lmer()`;
* generalized mixed-effects models with `lme4::glmer()`;
* conditional mixed models fitted with `glmmTMB`;
* repeated-measures ANOVA fitted through `afex::aov_ez()`;
* Cox proportional-hazards models with `survival::coxph()`;
* parametric survival models with `survival::survreg()`;
* custom fitting functions that return compatible model objects.

Packages such as `lme4`, `glmmTMB`, `afex`, and `survival` are required only for their corresponding model classes.

## Installation

Install the released version from CRAN:

```r
install.packages("statwitness")
```

Load the package:

```r
library(statwitness)
```

## General usage

The main interface is:

```r
audit <- statwitness(
  formula,
  data = your_data,
  focus = "term_of_interest",
  audit_level = "standard"
)
```

Print a concise certificate:

```r
audit
```

Display every individual check:

```r
print(audit, details = TRUE)
```

## Selecting the focus term

`focus` identifies the coefficient or omnibus model term treated as the primary result.

For example:

```r
audit <- statwitness(
  outcome ~ treatment + age + sex,
  data = dataset,
  focus = "treatment"
)
```

For an interaction:

```r
audit <- statwitness(
  outcome ~ treatment * sex,
  data = dataset,
  focus = "treatment:sex"
)
```

When `focus` refers to a multi-parameter term, such as an interaction between factors, `statwitness` evaluates it as an omnibus term rather than selecting a single dummy-coded coefficient.

The `focus` argument is optional. Specifying it is recommended whenever a model contains multiple predictors or interactions.

## Mixed-effects models

```r
audit <- statwitness(
  score ~ criterion * model + (1 | rater),
  data = dataset,
  focus = "criterion:model"
)

audit
```

For mixed-effects models, the standard audit can include:

* row-order invariance;
* analysis-sample reconstruction;
* fixed-effect prediction reconstruction;
* factor-label invariance;
* grouping-label invariance;
* outcome translation and scaling;
* fixed-effect design conditioning;
* convergence diagnostics;
* singularity screening;
* random-effect boundary checks;
* alternative-optimizer agreement;
* random-slope design screening;
* residual screening;
* leave-one-group-out influence analysis.

A possible certificate is:

```text
Computational integrity:    PASSED
Numerical stability:        PASSED
Design adequacy:            REVIEW NEEDED
Assumption screening:       PASSED
Influence stability:        PASSED

Overall status:
PASSED WITH REVIEW ITEMS
```

A design review does not automatically mean that the fitted model is incorrect. It identifies an aspect of the model that requires scientific consideration.

## Linear regression

```r
dat <- transform(
  mtcars,
  transmission = factor(am)
)

audit <- statwitness(
  mpg ~ transmission + wt + hp,
  data = dat,
  focus = "transmission"
)

audit
```

The standard linear-model audit can include:

* row-order invariance;
* formula-order invariance;
* outcome translation and scaling;
* predictor rescaling;
* model-matrix reconstruction;
* prediction reconstruction;
* fixed-design rank and conditioning;
* residual-shape screening;
* unequal-variance screening;
* Cook’s-distance influence screening.

## Logistic regression

```r
dat <- transform(
  mtcars,
  high_mpg = mpg > median(mpg),
  transmission = factor(am)
)

audit <- statwitness(
  high_mpg ~ transmission + wt,
  data = dat,
  family = binomial(),
  focus = "transmission"
)

audit
```

Generalized linear-model audits can additionally include:

* binary-outcome recoding;
* factor-reference reversal;
* linear-predictor reconstruction;
* likelihood verification;
* Pearson-dispersion screening;
* numerical separation screening.

## Factorial ANOVA

```r
audit <- statwitness(
  breaks ~ wool * tension,
  data = warpbreaks,
  focus = "wool:tension",
  method = "aov"
)

audit
```

The ANOVA design audit can identify:

* empty cells;
* unbalanced cells;
* rank-deficient terms;
* sensitivity to sequential term order;
* contrast-dependent interpretations.

## Repeated-measures ANOVA

Repeated-measures designs use a dedicated interface:

```r
audit <- statwitness_repeated(
  data = dataset,
  outcome = "score",
  id = "rater",
  within = c("criterion", "model"),
  focus = "criterion:model",
  type = 3,
  correction = "GG"
)

audit
```

The repeated-measures audit can include:

* participant-condition completeness;
* duplicated participant-condition observations;
* participant-label invariance;
* row-order invariance;
* outcome translation and scaling;
* sphericity-correction handling;
* residual screening;
* leave-one-participant-out influence analysis.

## Generalized mixed-effects models

```r
audit <- statwitness(
  correct ~ criterion * model + (1 | rater),
  data = dataset,
  family = binomial(),
  focus = "criterion:model"
)

audit
```

Generalized mixed-model audits combine GLM-specific checks with:

* convergence diagnostics;
* singularity screening;
* random-effect boundary checks;
* grouping-label invariance;
* optimizer agreement;
* group-level influence analysis.

## Cox proportional-hazards models

```r
audit <- statwitness(
  survival::Surv(time, event) ~ treatment + age,
  data = patients,
  focus = "treatment"
)

audit
```

The Cox-model audit can include:

* event-coding validation;
* analysis-sample reconstruction;
* time-unit scaling;
* event-information screening;
* proportional-hazards diagnostics;
* DFBETA influence screening.

## Audit levels

Three audit levels are available.

### Core

```r
statwitness(
  outcome ~ treatment,
  data = dataset,
  focus = "treatment",
  audit_level = "core"
)
```

Runs relatively fast computational-integrity checks.

### Standard

```r
statwitness(
  outcome ~ treatment,
  data = dataset,
  focus = "treatment",
  audit_level = "standard"
)
```

Adds routine numerical, design, assumption, and influence checks.

### Thorough

```r
statwitness(
  outcome ~ treatment,
  data = dataset,
  focus = "treatment",
  audit_level = "thorough"
)
```

Adds more computationally intensive checks, including broader optimizer comparisons and deletion-based influence analyses where supported.

The exact tests depend on the model class and analysis structure.

## Previewing an audit

Use `audit_plan()` to see which checks are applicable before running the full audit:

```r
audit_plan(
  score ~ criterion * model + (1 | rater),
  data = dataset,
  focus = "criterion:model",
  audit_level = "standard"
)
```

The plan reports:

* the selected checks;
* the domain of each check;
* checks that are not applicable;
* the reason each check was selected or skipped.

## Interpreting a certificate

Possible domain results include:

* `PASSED`;
* `WARNING`;
* `REVIEW NEEDED`;
* `FAILED`;
* `NOT EVALUATED`.

A passing certificate means that the analysis behaved as expected under the registered checks that were performed.

It does not establish that:

* the study design is correct;
* the model answers the intended scientific question;
* the estimand is appropriate;
* confounding is absent;
* causal interpretation is justified;
* the source data are accurate;
* every possible assumption has been satisfied;
* the effect is practically important.

`statwitness` complements scientific judgment and substantive model evaluation; it does not replace them.

## Reproducible certificates

Audit reports can include:

* a certificate identifier;
* a data fingerprint;
* an analysis fingerprint;
* the model formula;
* the model class;
* the selected focus term;
* the R and package versions;
* the checks performed;
* numerical tolerances used by the checks.

Fingerprints help determine whether two certificates were generated from the same supplied data and analysis configuration.

## Citation

To cite `statwitness` in publications, run:

```r
citation("statwitness")
```

library(statwitness)

dat <- transform(
  mtcars,
  transmission = factor(am, labels = c("automatic", "manual"))
)

plan <- audit_plan(
  mpg ~ transmission + wt + hp,
  data = dat,
  focus = transmission
)
print(plan)

audit <- statwitness(
  mpg ~ transmission + wt + hp,
  data = dat,
  focus = transmission
)
print(audit, details = TRUE)
summary(audit)

.test_group_label_invariance <- function(info, variable, tolerance) {
  d2 <- info$data
  x <- factor(d2[[variable]])
  levels(x) <- paste0("SW_GROUP_", seq_along(levels(x)))
  d2[[variable]] <- x
  m2 <- .refit(info, d2)
  ov <- .named_overlap(.fixed_coef(info$model), .fixed_coef(m2))
  ll1 <- .model_loglik(info$model)
  ll2 <- .model_loglik(m2)
  ok <- .within_tol(ov$x, ov$y, tolerance * 100) && .within_tol(ll1, ll2, tolerance * 100)
  .new_test(
    "group_label_invariance", paste0("Grouping-label invariance: ", variable), if (ok) "PASS" else "FAIL",
    expected = "Renaming cluster labels preserves fixed effects and likelihood.",
    observed = paste0("maximum fixed-effect difference = ", .format_num(.max_abs_diff(ov$x, ov$y)),
                      "; likelihood difference = ", .format_num(abs(ll1 - ll2))),
    details = if (ok) "Random-effect grouping labels were treated as identifiers." else "Renaming grouping labels changed the mixed model."
  )
}

.test_mixed_convergence <- function(info) {
  model <- info$model
  status <- "PASS"
  observed <- "No convergence problem was reported."
  details <- "The optimizer diagnostics did not report a convergence failure."
  if (info$model_class %in% c("lmerMod", "glmerMod")) {
    msgs <- model@optinfo$conv$lme4$messages
    code <- model@optinfo$conv$opt
    if (!is.null(msgs) || (!is.null(code) && any(code != 0))) {
      status <- "WARN"
      observed <- paste(c(msgs, paste0("optimizer code: ", paste(code, collapse = ", "))), collapse = "; ")
      details <- "The mixed model reported a convergence warning; estimates require review."
    }
  } else if (info$model_class == "glmmTMB") {
    code <- info$model$fit$convergence
    pdh <- tryCatch(info$model$sdr$pdHess, error = function(e) NA)
    if (!identical(code, 0L) || isFALSE(pdh)) {
      status <- "WARN"
      observed <- paste0("optimizer code = ", code, "; positive-definite Hessian = ", pdh)
      details <- "The glmmTMB fit reported convergence or Hessian concerns."
    }
  }
  .new_test(
    "mixed_convergence", "Mixed-model convergence audit", status, severity = "warning",
    expected = "A successful optimizer termination without unresolved convergence messages.",
    observed = observed, details = details
  )
}

.test_mixed_singularity <- function(info, tolerance) {
  if (info$model_class %in% c("lmerMod", "glmerMod")) {
    singular <- lme4::isSingular(info$model, tol = max(tolerance, 1e-5))
    return(.new_test(
      "mixed_singularity", "Random-effects singularity audit", if (singular) "WARN" else "PASS",
      severity = "warning",
      expected = "The requested random-effects covariance structure is supported away from its boundary.",
      observed = paste0("singular fit = ", singular),
      details = if (singular) "At least one random-effects dimension is on or near the boundary; the random structure may be more complex than the data support." else "No singular random-effects structure was detected."
    ))
  }
  vc <- tryCatch(glmmTMB::VarCorr(info$model), error = function(e) NULL)
  near_zero <- FALSE
  if (!is.null(vc)) {
    vals <- suppressWarnings(as.numeric(unlist(vc)))
    vals <- vals[is.finite(vals)]
    near_zero <- length(vals) && any(abs(vals) < max(tolerance, 1e-8))
  }
  .new_test(
    "mixed_singularity", "Random-effects boundary audit", if (near_zero) "WARN" else "PASS",
    severity = "warning",
    expected = "No estimated random-effect variance lies effectively at zero.",
    observed = paste0("near-zero variance detected = ", near_zero),
    details = if (near_zero) "A random-effect variance appears to be on the boundary." else "No obvious random-effect boundary estimate was detected."
  )
}

.test_optimizer_agreement <- function(info, tolerance) {
  if (!info$model_class %in% c("lmerMod", "glmerMod")) {
    return(.new_test("optimizer_agreement", "Alternative-optimizer agreement", "WARN", severity = "warning",
      expected = "A registered alternative optimizer is available.", observed = "No alternative optimizer adapter is registered.",
      details = "Optimizer agreement is currently implemented for lme4 models."))
  }
  if (info$model_class == "lmerMod") {
    ctrl <- lme4::lmerControl(optimizer = "bobyqa")
  } else {
    ctrl <- lme4::glmerControl(optimizer = "bobyqa", optCtrl = list(maxfun = 2e5))
  }
  m2 <- .refit(info, info$data, control = ctrl)
  ov <- .named_overlap(.fixed_coef(info$model), .fixed_coef(m2))
  ll1 <- .model_loglik(info$model)
  ll2 <- .model_loglik(m2)
  ok <- .within_tol(ov$x, ov$y, max(tolerance * 1000, 1e-5)) &&
    .within_tol(ll1, ll2, max(tolerance * 1000, 1e-5))
  .new_test(
    "optimizer_agreement", "Alternative-optimizer agreement", if (ok) "PASS" else "WARN",
    severity = "warning",
    expected = "A second optimizer reaches materially equivalent fixed effects and likelihood.",
    observed = paste0("maximum fixed-effect difference = ", .format_num(.max_abs_diff(ov$x, ov$y)),
                      "; likelihood difference = ", .format_num(abs(ll1 - ll2))),
    details = if (ok) "The alternative optimizer reached an equivalent solution." else "The alternative optimizer reached a materially different solution or local optimum."
  )
}

.test_random_structure <- function(info) {
  if (!info$model_class %in% c("lmerMod", "glmerMod")) {
    return(.new_test("random_structure", "Random-effects structure summary", "PASS", severity = "warning",
      expected = "Random-effect variance information is available.", observed = "Model-specific variance information recorded.",
      details = "Review the model object's variance-covariance summary for boundary estimates."))
  }
  vc <- as.data.frame(lme4::VarCorr(info$model))
  zero <- vc$vcov < 1e-8
  corr_boundary <- !is.na(vc$var2) & abs(vc$sdcor) > 0.999
  issue <- any(zero | corr_boundary)
  observed <- paste0("variance components = ", nrow(vc),
                     "; near-zero components = ", sum(zero),
                     "; boundary correlations = ", sum(corr_boundary))
  .new_test(
    "random_structure", "Random-effects structure summary", if (issue) "WARN" else "PASS",
    severity = "warning",
    expected = "No near-zero variance component or near-perfect random-effect correlation.",
    observed = observed,
    details = if (issue) "The random-effects structure contains a boundary-like component that merits review." else "No obvious boundary-like random-effect component was found."
  )
}

.status_symbol <- function(status) {
  switch(status,
    PASS = "[PASS]", PASSED = "[PASS]",
    WARN = "[WARN]", REVIEW = "[REVIEW]", WARNING = "[WARN]",
    FAIL = "[FAIL]", FAILED = "[FAIL]", ERROR = "[ERROR]",
    `NOT EVALUATED` = "[N/A]", paste0("[", status, "]")
  )
}

.domain_label <- function(x) {
  switch(x,
    computational_integrity = "Computational integrity",
    numerical_stability = "Numerical stability",
    design_adequacy = "Design adequacy",
    assumption_screening = "Assumption screening",
    influence_stability = "Influence stability",
    other = "Other checks",
    x
  )
}

#' @export
print.statwitness_audit <- function(x, ..., details = FALSE) {
  cat("STATWITNESS VALIDATION CERTIFICATE\n")
  cat(strrep("=", 60), "\n", sep = "")
  cat("Model type:   ", x$model_type, "\n", sep = "")
  if (!is.null(x$formula)) cat("Formula:      ", paste(deparse(x$formula), collapse = " "), "\n", sep = "")
  cat("Focus:        ", x$primary$label, if (x$focus_inferred) " (inferred)" else "", "\n", sep = "")
  cat("Observations: ", x$primary$n, "\n", sep = "")
  if (!is.null(x$audit_level)) cat("Audit level:  ", x$audit_level, "\n", sep = "")

  cat("\nPRIMARY RESULT\n")
  cat(strrep("-", 60), "\n", sep = "")
  if (identical(x$primary$kind, "term")) {
    cat("Omnibus term: ", x$primary$label, "\n", sep = "")
    cat("Test method:  ", x$primary$method, "\n", sep = "")
    cat("Statistic:    ", .format_num(x$primary$statistic), "\n", sep = "")
    if (length(x$primary$df) == 2L) {
      cat("Degrees df:   ", paste(.format_num(x$primary$df), collapse = ", "), "\n", sep = "")
    } else if (length(x$primary$df) == 1L && !is.na(x$primary$df)) {
      cat("Degrees df:   ", .format_num(x$primary$df), "\n", sep = "")
    }
    if (!is.na(x$primary$estimate)) {
      label <- x$primary$effect_size_label %||% "Effect-size estimate"
      cat(label, ": ", .format_num(x$primary$estimate), "\n", sep = "")
    }
    cat("p-value:      ", format.pval(x$primary$p_value, digits = 4, eps = 0.0001), "\n", sep = "")
  } else {
    cat("Estimate:     ", .format_num(x$primary$estimate), "\n", sep = "")
    cat(round(100 * x$primary$conf_level), "% CI:      [", .format_num(x$primary$lower), ", ", .format_num(x$primary$upper), "]\n", sep = "")
    cat("p-value:      ", format.pval(x$primary$p_value, digits = 4, eps = 0.0001), "\n", sep = "")
    if (!is.null(x$primary$transformed)) {
      tr <- x$primary$transformed
      cat(tr$label, ": ", .format_num(tr$estimate), " [", .format_num(tr$lower), ", ", .format_num(tr$upper), "]\n", sep = "")
    }
  }

  domains <- x$domains %||% .domain_summary(x$tests)
  cat("\nAUDIT DOMAINS\n")
  cat(strrep("-", 60), "\n", sep = "")
  for (i in seq_len(nrow(domains))) {
    cat(.status_symbol(domains$status[[i]]), " ", .domain_label(domains$domain[[i]]),
        " - ", domains$passed[[i]], " passed, ", domains$review[[i]], " review item(s)\n", sep = "")
  }

  show <- if (isTRUE(details)) rep(TRUE, nrow(x$tests)) else x$tests$status != "PASS"
  if (any(show)) {
    cat("\n", if (isTRUE(details)) "ALL REGISTERED TESTS" else "FINDINGS REQUIRING REVIEW", "\n", sep = "")
    cat(strrep("-", 60), "\n", sep = "")
    idx <- which(show)
    for (i in idx) {
      cat(.status_symbol(x$tests$status[[i]]), " ", x$tests$test[[i]],
          " [", .domain_label(x$tests$domain[[i]]), "]\n", sep = "")
      cat("  ", x$tests$observed[[i]], "\n", sep = "")
      if (nzchar(x$tests$details[[i]])) cat("  ", x$tests$details[[i]], "\n", sep = "")
    }
  } else {
    cat("\nAll registered tests passed. Use `print(x, details = TRUE)` for the complete audit trail.\n")
  }

  cat("\nCERTIFICATE STATUS: ", x$status, "\n", sep = "")
  cat("Certificate ID:     ", x$certificate_id, "\n", sep = "")
  cat("Data fingerprint:   ", x$data_fingerprint, "\n", sep = "")
  cat("Analysis fingerprint: ", x$analysis_fingerprint, "\n", sep = "")
  cat(strrep("-", 60), "\n", sep = "")
  cat(x$limitations[[1L]], "\n")
  invisible(x)
}

#' @export
summary.statwitness_audit <- function(object, ...) {
  counts <- table(factor(object$tests$status, levels = c("PASS", "WARN", "FAIL", "ERROR")))
  structure(list(
    status = object$status, model_type = object$model_type,
    focus = object$primary$label, primary = object$primary,
    counts = counts, domains = object$domains %||% .domain_summary(object$tests),
    failures = object$tests[object$tests$status %in% c("WARN", "FAIL", "ERROR"), , drop = FALSE]
  ), class = "summary.statwitness_audit")
}

#' @export
print.summary.statwitness_audit <- function(x, ...) {
  cat("statwitness summary\n")
  cat("Certificate: ", x$status, "\n", sep = "")
  cat("Model:       ", x$model_type, "\n", sep = "")
  cat("Focus:       ", x$focus, "\n", sep = "")
  if (x$primary$kind == "term") cat("Statistic:   ", .format_num(x$primary$statistic), "\n", sep = "") else
    cat("Estimate:    ", .format_num(x$primary$estimate), "\n", sep = "")
  cat("Tests:       ", paste(names(x$counts), as.integer(x$counts), sep = "=", collapse = ", "), "\n", sep = "")
  cat("\nDomains:\n")
  for (i in seq_len(nrow(x$domains))) {
    cat("- ", .domain_label(x$domains$domain[[i]]), ": ", x$domains$status[[i]], "\n", sep = "")
  }
  if (nrow(x$failures)) {
    cat("\nNon-passing findings:\n")
    for (i in seq_len(nrow(x$failures))) cat("- ", x$failures$test[[i]], ": ", x$failures$details[[i]], "\n", sep = "")
  }
  invisible(x)
}

#' @export
plot.statwitness_audit <- function(x, ...) {
  statuses <- factor(x$tests$status, levels = c("PASS", "WARN", "FAIL", "ERROR"))
  counts <- table(statuses)
  graphics::barplot(counts, main = paste("statwitness certificate:", x$status),
                    ylab = "Number of tests", xlab = "Test status", ...)
  invisible(counts)
}

.test_outcome_translation <- function(info, tolerance) {
  yname <- info$response_name
  d2 <- info$data
  y <- d2[[yname]]
  shift <- stats::sd(y, na.rm = TRUE)
  if (!is.finite(shift) || shift == 0) shift <- 1
  d2[[yname]] <- y + shift
  m2 <- .refit(info, d2)
  expected <- .fixed_coef(info$model)
  if ("(Intercept)" %in% names(expected)) expected[["(Intercept)"]] <- expected[["(Intercept)"]] + shift
  observed <- .fixed_coef(m2)
  ov <- .named_overlap(expected, observed)
  sigma1 <- .model_sigma(info$model)
  sigma2 <- .model_sigma(m2)
  sd1 <- .random_sd_vector(info$model)
  sd2 <- .random_sd_vector(m2)
  sdov <- .named_overlap(sd1, sd2)
  ok_sigma <- if (is.na(sigma1) || is.na(sigma2)) TRUE else .within_tol(sigma1, sigma2, tolerance * 100)
  ok_random <- if (!length(sdov$x)) TRUE else .within_tol(sdov$x, sdov$y, tolerance * 100)
  ok <- .within_tol(ov$x, ov$y, tolerance * 100) && ok_sigma && ok_random
  .new_test(
    "outcome_translation", "Outcome translation", if (ok) "PASS" else "FAIL",
    expected = "Adding a constant changes only the fixed intercept and leaves scale parameters unchanged.",
    observed = paste0("maximum fixed-effect difference from expectation = ", .format_num(.max_abs_diff(ov$x, ov$y)),
                      "; residual-scale difference = ", .format_num(abs(sigma1 - sigma2)),
                      "; maximum random-SD difference = ", .format_num(.max_abs_diff(sdov$x, sdov$y))),
    details = if (ok) "Outcome translation propagated correctly." else "The model did not respond correctly to outcome translation."
  )
}

.test_outcome_scaling <- function(info, tolerance) {
  yname <- info$response_name
  d2 <- info$data
  d2[[yname]] <- d2[[yname]] * 10
  m2 <- .refit(info, d2)
  expected <- .fixed_coef(info$model) * 10
  observed <- .fixed_coef(m2)
  ov <- .named_overlap(expected, observed)
  p1 <- .model_p_values(info$model)
  p2 <- .model_p_values(m2)
  pv <- .named_overlap(p1, p2)
  sigma1 <- .model_sigma(info$model)
  sigma2 <- .model_sigma(m2)
  sd1 <- .random_sd_vector(info$model)
  sd2 <- .random_sd_vector(m2)
  sdov <- .named_overlap(10 * sd1, sd2)
  ok_sigma <- if (is.na(sigma1) || is.na(sigma2)) TRUE else .within_tol(10 * sigma1, sigma2, tolerance * 100)
  ok_random <- if (!length(sdov$x)) TRUE else .within_tol(sdov$x, sdov$y, tolerance * 100)
  ok <- .within_tol(ov$x, ov$y, tolerance * 100) && .within_tol(pv$x, pv$y, tolerance * 100) && ok_sigma && ok_random
  .new_test(
    "outcome_scaling", "Outcome-unit scaling", if (ok) "PASS" else "FAIL",
    expected = "Fixed effects and Gaussian scale parameters multiply by 10 while p-values remain unchanged.",
    observed = paste0("maximum fixed-effect difference = ", .format_num(.max_abs_diff(ov$x, ov$y)),
                      "; maximum p-value difference = ", .format_num(.max_abs_diff(pv$x, pv$y)),
                      "; residual-scale difference = ", .format_num(abs(10 * sigma1 - sigma2)),
                      "; maximum random-SD difference = ", .format_num(.max_abs_diff(sdov$x, sdov$y))),
    details = if (ok) "Outcome-unit conversion propagated correctly." else "Outcome scaling did not propagate as expected."
  )
}

.test_known_effect <- function(info, tolerance) {
  d2 <- info$data
  yname <- info$response_name
  y <- d2[[yname]]
  delta <- stats::sd(y, na.rm = TRUE) / 2
  if (!is.finite(delta) || delta == 0) delta <- 1
  if (info$focus_binary_factor) {
    xfac <- factor(d2[[info$focus]])
    x <- as.numeric(xfac == levels(xfac)[[2L]])
  } else {
    x <- d2[[info$focus]]
  }
  d2[[yname]] <- y + delta * x
  m2 <- .refit(info, d2)
  expected <- .fixed_coef(info$model)
  expected[[info$focus_coef]] <- expected[[info$focus_coef]] + delta
  observed <- .fixed_coef(m2)
  ov <- .named_overlap(expected, observed)
  ok <- .within_tol(ov$x, ov$y, tolerance * 100)
  .new_test(
    "known_effect_injection", "Known-effect injection", if (ok) "PASS" else "FAIL",
    expected = paste0("Injecting ", .format_num(delta), " units adds that amount to the focus coefficient."),
    observed = paste0("expected focus coefficient = ", .format_num(expected[[info$focus_coef]]),
                      "; observed = ", .format_num(observed[[info$focus_coef]])),
    details = if (ok) "The known injected linear effect was recovered." else "The injected effect was not recovered as expected."
  )
}

.test_independent_qr <- function(info, tolerance) {
  model <- info$model
  X <- stats::model.matrix(model)
  y <- stats::model.response(stats::model.frame(model))
  if (!is.null(model$offset)) y <- y - model$offset
  w <- .effective_model_weights(model)
  if (is.null(w)) w <- rep(1, nrow(X))
  sw <- sqrt(w)
  q <- qr(X * sw)
  beta <- qr.coef(q, y * sw)
  names(beta) <- colnames(X)
  ov <- .named_overlap(stats::coef(model), beta)
  ok <- .within_tol(ov$x, ov$y, tolerance * 10)
  .new_test(
    "independent_qr", "Independent weighted-QR coefficient calculation", if (ok) "PASS" else "FAIL",
    expected = "Coefficients match a separately executed weighted QR solution.",
    observed = paste0("maximum coefficient difference = ", .format_num(.max_abs_diff(ov$x, ov$y))),
    details = if (ok) "The coefficient vector was independently reconstructed." else "The fitted coefficients disagree with the weighted QR calculation."
  )
}

.test_anova_cells <- function(info) {
  vars <- info$factor_predictors
  tab <- do.call(table, c(info$data[vars], useNA = "no"))
  values <- as.numeric(tab)
  empty <- sum(values == 0)
  positive <- values[values > 0]
  ratio <- if (length(positive)) max(positive) / min(positive) else Inf
  status <- if (empty > 0) "WARN" else "PASS"
  .new_test(
    "anova_cell_audit", "ANOVA cell-structure audit", status, severity = "warning",
    expected = "All factor combinations are represented; imbalance is reported transparently.",
    observed = paste0("empty cells = ", empty, "; largest/smallest non-empty cell ratio = ", .format_num(ratio)),
    details = if (empty > 0) "At least one design cell is empty; omnibus effects may depend strongly on hypothesis parameterization." else "No empty design cells were found. Cell-size imbalance is reported but not judged automatically."
  )
}

.test_anova_term_order <- function(info, tolerance) {
  if (info$focus_kind != "term") {
    return(.new_test("anova_term_order", "ANOVA term-order sensitivity", "WARN", severity = "warning",
      expected = "An omnibus focus term is supplied.", observed = "The focus is coefficient-level.",
      details = "Term-order sensitivity is most meaningful for omnibus ANOVA terms."))
  }
  get_sequential_p <- function(model, term) {
    tab <- stats::anova(model)
    idx <- match(term, rownames(tab))
    pcol <- grep("Pr\\(", colnames(tab), value = TRUE)
    if (is.na(idx) || !length(pcol)) return(NA_real_)
    unname(tab[idx, pcol[[1L]]])
  }
  f2 <- .reordered_formula(info$formula)
  m2 <- .refit(info, info$data, formula = f2)
  p1 <- get_sequential_p(info$model, info$focus_term)
  p2 <- get_sequential_p(m2, info$focus_term)
  diff <- abs(p1 - p2)
  status <- if (is.na(diff) || diff <= tolerance * 100) "PASS" else "WARN"
  .new_test(
    "anova_term_order", "ANOVA term-order sensitivity", status, severity = "warning",
    expected = "The sequential omnibus result is stable to an equivalent term ordering.",
    observed = paste0("original sequential p-value = ", .format_num(p1), "; reordered p-value = ", .format_num(p2)),
    details = if (status == "PASS") "No material Type-I term-order sensitivity was detected." else "The sequential ANOVA result changed with term ordering; report the sums-of-squares and hypothesis convention explicitly."
  )
}

.model_class_id <- function(model) {
  if (inherits(model, "glmerMod")) return("glmerMod")
  if (inherits(model, "lmerMod")) return("lmerMod")
  if (inherits(model, "glmmTMB")) return("glmmTMB")
  if (inherits(model, "coxph")) return("coxph")
  if (inherits(model, "survreg")) return("survreg")
  if (inherits(model, "glm")) return("glm")
  if (inherits(model, "aov")) return("aov")
  if (inherits(model, "lm")) return("lm")
  class(model)[[1L]]
}

.fixed_coef <- function(model) {
  id <- .model_class_id(model)
  if (id %in% c("lmerMod", "glmerMod")) return(lme4::fixef(model))
  if (id == "glmmTMB") return(glmmTMB::fixef(model)$cond)
  stats::coef(model)
}

.fixed_vcov <- function(model) {
  id <- .model_class_id(model)
  if (id == "glmmTMB") return(as.matrix(stats::vcov(model)$cond))
  as.matrix(stats::vcov(model))
}

.model_frame_safe <- function(model) {
  out <- tryCatch(stats::model.frame(model), error = function(e) NULL)
  if (!is.null(out)) return(out)
  if (inherits(model, "glmmTMB")) {
    out <- tryCatch(model$frame, error = function(e) NULL)
  }
  out
}

.model_matrix_fixed <- function(model) {
  id <- .model_class_id(model)
  if (id == "glmmTMB") return(stats::model.matrix(model, component = "cond"))
  stats::model.matrix(model)
}

.model_nobs <- function(model) {
  as.integer(stats::nobs(model))
}

.model_loglik <- function(model) {
  tryCatch(as.numeric(stats::logLik(model)), error = function(e) NA_real_)
}

.model_fixed_prediction <- function(model) {
  id <- .model_class_id(model)
  if (id == "lmerMod") return(as.numeric(stats::predict(model, re.form = NA)))
  if (id == "glmerMod") return(as.numeric(stats::predict(model, re.form = NA, type = "link")))
  if (id == "glmmTMB") return(as.numeric(stats::predict(model, re.form = NA, type = "link")))
  if (id == "coxph") return(as.numeric(stats::predict(model, type = "lp")))
  if (id == "survreg") return(as.numeric(stats::predict(model, type = "lp")))
  if (id == "glm") return(as.numeric(stats::predict(model, type = "link")))
  as.numeric(stats::predict(model))
}

.model_response_prediction <- function(model) {
  id <- .model_class_id(model)
  if (id == "glmerMod") return(as.numeric(stats::predict(model, type = "response", re.form = NULL)))
  if (id == "glmmTMB") return(as.numeric(stats::predict(model, type = "response", re.form = NULL)))
  if (id == "glm") return(as.numeric(stats::predict(model, type = "response")))
  as.numeric(stats::fitted(model))
}

.model_residuals <- function(model) {
  tryCatch(as.numeric(stats::residuals(model)), error = function(e) numeric())
}

.model_sigma <- function(model) {
  id <- .model_class_id(model)
  if (id == "lmerMod") return(stats::sigma(model))
  if (id == "glmmTMB") return(stats::sigma(model))
  if (id %in% c("lm", "aov")) return(sqrt(sum(stats::residuals(model)^2) / stats::df.residual(model)))
  NA_real_
}

.model_p_values <- function(model) {
  id <- .model_class_id(model)
  sm <- summary(model)
  tab <- sm$coefficients
  if (id == "glmmTMB") tab <- sm$coefficients$cond
  if (is.null(tab) || is.null(dim(tab))) {
    return(setNames(rep(NA_real_, length(.fixed_coef(model))), names(.fixed_coef(model))))
  }
  pcol <- grep("Pr\\(", colnames(tab), value = TRUE)
  if (!length(pcol)) {
    z <- tab[, min(3L, ncol(tab))]
    return(setNames(2 * stats::pnorm(abs(z), lower.tail = FALSE), rownames(tab)))
  }
  setNames(tab[, pcol[[1L]]], rownames(tab))
}

.random_group_vars <- function(formula) {
  if (!.has_random_terms(formula)) return(character())
  .require_namespace("reformulas", "mixed-model formula parsing")
  bars <- reformulas::findbars(formula)
  unique(unlist(lapply(bars, function(x) all.vars(x[[3L]]))))
}

.response_variables <- function(formula) {
  all.vars(formula[[2L]])
}

.analysis_variables <- function(formula) {
  unique(all.vars(formula))
}

.model_family_info <- function(model) {
  id <- .model_class_id(model)
  if (id == "glm") return(list(family = model$family$family, link = model$family$link, object = model$family))
  if (id == "glmerMod") {
    fam <- stats::family(model)
    return(list(family = fam$family, link = fam$link, object = fam))
  }
  if (id == "glmmTMB") {
    fam <- stats::family(model)
    return(list(family = fam$family, link = fam$link, object = fam))
  }
  if (id == "coxph") return(list(family = "cox", link = "log-hazard", object = NULL))
  if (id == "survreg") return(list(family = paste0("survreg_", model$dist), link = "location-scale", object = NULL))
  list(family = "gaussian", link = "identity", object = NULL)
}

.random_sd_vector <- function(model) {
  id <- .model_class_id(model)
  if (id %in% c("lmerMod", "glmerMod")) {
    vc <- as.data.frame(lme4::VarCorr(model))
    keep <- is.na(vc$var2) & vc$grp != "Residual"
    out <- vc$sdcor[keep]
    names(out) <- paste(vc$grp[keep], vc$var1[keep], sep = ":")
    return(out)
  }
  numeric()
}

.is_gaussian_identity <- function(info) {
  info$family == "gaussian" && info$link == "identity"
}

# A deliberately incorrect engine that accepts weights but ignores them.
bad_engine <- function(formula, data, family = NULL, weights = NULL) {
  if (is.null(family)) {
    stats::lm(formula, data = data, model = TRUE, x = TRUE, y = TRUE)
  } else {
    stats::glm(formula, data = data, family = family, model = TRUE, x = TRUE, y = TRUE)
  }
}

weighted_data <- transform(mtcars, audit_weight = seq_len(nrow(mtcars)))

# The weight-semantics test should fail.
statwitness::statwitness(
  mpg ~ wt + hp,
  data = weighted_data,
  focus = "wt",
  weights = audit_weight,
  engine = bad_engine
)

`%||%` <- function(x, y) {
  if (is.null(x)) y else x
}

.sw_abort <- function(..., call. = FALSE) {
  stop(paste0(...), call. = call.)
}

.require_namespace <- function(package, feature = package) {
  if (!requireNamespace(package, quietly = TRUE)) {
    .sw_abort("Package `", package, "` is required for ", feature, ". Install it with install.packages(\"", package, "\").")
  }
  invisible(TRUE)
}

.within_tol <- function(x, y, tolerance = 1e-7) {
  if (length(x) != length(y)) return(FALSE)
  if (length(x) == 0L) return(TRUE)
  x <- as.numeric(x)
  y <- as.numeric(y)
  ok_na <- is.na(x) == is.na(y)
  if (!all(ok_na)) return(FALSE)
  keep <- !is.na(x) & !is.na(y)
  if (!any(keep)) return(TRUE)
  all(abs(x[keep] - y[keep]) <= tolerance * (1 + pmax(abs(x[keep]), abs(y[keep]))))
}

.max_abs_diff <- function(x, y) {
  if (length(x) != length(y)) return(Inf)
  x <- as.numeric(x)
  y <- as.numeric(y)
  keep <- !is.na(x) & !is.na(y)
  if (!any(keep)) return(0)
  max(abs(x[keep] - y[keep]))
}

.named_overlap <- function(x, y) {
  nms <- intersect(names(x), names(y))
  list(x = x[nms], y = y[nms], names = nms)
}

.resolve_weights <- function(data, weights, weights_expr, weights_missing) {
  if (weights_missing) return(NULL)
  if (is.symbol(weights_expr)) {
    nm <- as.character(weights_expr)
    if (nm %in% names(data)) weights <- data[[nm]]
  }
  if (is.character(weights) && length(weights) == 1L && weights %in% names(data)) {
    weights <- data[[weights]]
  }
  if (is.null(weights)) return(NULL)
  if (!is.numeric(weights)) {
    .sw_abort("`weights` must be NULL, a numeric vector, or the name of a numeric column.")
  }
  if (length(weights) != nrow(data)) {
    .sw_abort("`weights` must have one value per row of `data`.")
  }
  if (any(weights < 0, na.rm = TRUE)) {
    .sw_abort("`weights` cannot contain negative values.")
  }
  weights
}

.has_random_terms <- function(formula) {
  grepl("\\|", paste(deparse(formula), collapse = " "))
}

.is_survival_formula <- function(formula) {
  lhs <- paste(deparse(formula[[2L]]), collapse = "")
  grepl("(^|::)Surv\\s*\\(", lhs)
}

.fixed_formula <- function(formula) {
  if (.has_random_terms(formula)) {
    .require_namespace("reformulas", "mixed-model formula parsing")
    out <- reformulas::nobars(formula)
    environment(out) <- environment(formula)
    return(out)
  }
  formula
}

.detect_method <- function(formula, family = NULL, method = "auto") {
  allowed <- c("auto", "lm", "aov", "glm", "lmer", "glmer", "glmmTMB", "coxph", "survreg")
  method <- match.arg(method, allowed)
  if (method != "auto") return(method)
  if (.is_survival_formula(formula)) return("coxph")
  if (.has_random_terms(formula)) {
    if (is.null(family)) return("lmer")
    return("glmer")
  }
  if (!is.null(family)) return("glm")
  "lm"
}

.call_engine <- function(engine, args) {
  if (!is.function(engine)) .sw_abort("`engine` must be NULL or a function.")
  fmls <- names(formals(engine))
  if (!"..." %in% fmls) args <- args[intersect(names(args), fmls)]
  do.call(engine, args)
}

.default_engine <- function(formula, data, family = NULL, weights = NULL,
                            method = "auto", control = NULL, ...) {
  method <- .detect_method(formula, family, method)
  add_weights <- function(args) {
    if (!is.null(weights)) args$weights <- weights
    args
  }
  if (method == "lm") {
    args <- add_weights(list(formula = formula, data = data,
      na.action = stats::na.omit, model = TRUE, x = TRUE, y = TRUE))
    return(do.call(stats::lm, args))
  }
  if (method == "aov") {
    args <- add_weights(list(formula = formula, data = data, na.action = stats::na.omit))
    return(do.call(stats::aov, args))
  }
  if (method == "glm") {
    args <- add_weights(list(formula = formula, data = data, family = family,
      na.action = stats::na.omit, model = TRUE, x = TRUE, y = TRUE))
    return(do.call(stats::glm, args))
  }
  if (method == "lmer") {
    .require_namespace("lme4", "linear mixed-effects models")
    args <- add_weights(list(formula = formula, data = data, REML = TRUE,
      na.action = stats::na.omit))
    if (!is.null(control)) args$control <- control
    return(do.call(lme4::lmer, args))
  }
  if (method == "glmer") {
    .require_namespace("lme4", "generalized mixed-effects models")
    args <- add_weights(list(formula = formula, data = data, family = family,
      na.action = stats::na.omit))
    if (!is.null(control)) args$control <- control
    return(do.call(lme4::glmer, args))
  }
  if (method == "glmmTMB") {
    .require_namespace("glmmTMB", "glmmTMB models")
    args <- add_weights(list(formula = formula, data = data,
      family = family %||% stats::gaussian(), na.action = stats::na.omit))
    if (!is.null(control)) args$control <- control
    return(do.call(glmmTMB::glmmTMB, args))
  }
  if (method == "coxph") {
    .require_namespace("survival", "Cox proportional-hazards models")
    args <- add_weights(list(formula = formula, data = data,
      na.action = stats::na.omit, model = TRUE, x = TRUE, y = TRUE))
    return(do.call(survival::coxph, args))
  }
  if (method == "survreg") {
    .require_namespace("survival", "parametric survival models")
    args <- add_weights(list(formula = formula, data = data,
      na.action = stats::na.omit, model = TRUE, x = TRUE, y = TRUE))
    return(do.call(survival::survreg, args))
  }
  .sw_abort("Unsupported method: ", method)
}

.fit_analysis <- function(formula, data, family = NULL, weights = NULL,
                          method = "auto", engine = NULL, control = NULL) {
  method <- .detect_method(formula, family, method)
  engine <- engine %||% .default_engine
  model <- .call_engine(engine, list(
    formula = formula, data = data, family = family, weights = weights,
    method = method, control = control
  ))
  supported <- inherits(model, c("lm", "glm", "aov", "merMod", "glmmTMB", "coxph", "survreg"))
  if (!supported) {
    .sw_abort("The fitting engine returned an unsupported model class: ", paste(class(model), collapse = ", "), ".")
  }
  model
}

.refit <- function(info, data, formula = info$formula, weights = info$weights,
                   control = info$control) {
  .fit_analysis(
    formula = formula,
    data = data,
    family = info$family_object,
    weights = weights,
    method = info$method,
    engine = info$engine,
    control = control
  )
}

.format_num <- function(x, digits = 4L) {
  if (length(x) == 0L || is.na(x)) return("NA")
  formatC(x, digits = digits, format = "fg", flag = "#")
}

.test_domain <- function(id) {
  computational <- c(
    "row_order", "formula_order", "model_frame", "prediction_reconstruction",
    "factor_label_invariance", "predictor_rescaling", "focus_reference_reversal",
    "outcome_translation", "outcome_scaling", "known_effect_injection",
    "independent_qr", "binary_outcome_reversal", "likelihood_reconstruction",
    "weight_semantics", "group_label_invariance", "survival_time_scaling",
    "event_coding", "rm_row_order", "rm_id_label", "rm_outcome_translation",
    "rm_outcome_scaling"
  )
  numerical <- c(
    "mixed_convergence", "mixed_singularity", "optimizer_agreement",
    "random_structure", "model_convergence", "fixed_design_condition"
  )
  design <- c(
    "design_support", "anova_cell_audit", "anova_term_order", "mixed_design_adequacy",
    "survival_information", "rm_design_integrity", "rm_sphericity"
  )
  assumptions <- c(
    "residual_shape", "residual_variance", "glm_dispersion",
    "binomial_separation", "cox_ph_assumption", "rm_residual_shape"
  )
  influence <- c(
    "case_influence", "group_influence", "survival_influence", "rm_id_influence"
  )
  if (id %in% computational) return("computational_integrity")
  if (id %in% numerical) return("numerical_stability")
  if (id %in% design) return("design_adequacy")
  if (id %in% assumptions) return("assumption_screening")
  if (id %in% influence) return("influence_stability")
  "other"
}

.new_test <- function(id, label, status, severity = "critical",
                      expected = "", observed = "", details = "",
                      domain = .test_domain(id)) {
  list(id = id, label = label, status = status, severity = severity,
       domain = domain, expected = expected, observed = observed, details = details)
}

.test_to_row <- function(x) {
  data.frame(id = x$id, test = x$label, status = x$status,
             severity = x$severity, domain = x$domain %||% .test_domain(x$id),
             expected = x$expected, observed = x$observed, details = x$details,
             stringsAsFactors = FALSE)
}

.safe_test <- function(expr, id, label) {
  tryCatch(expr, error = function(e) {
    .new_test(id, label, "ERROR", "critical",
              expected = "The validation experiment should run successfully.",
              observed = conditionMessage(e),
              details = "The audit implementation could not complete this test.")
  })
}

.md5_object <- function(x) {
  path <- tempfile(fileext = ".rds")
  on.exit(unlink(path), add = TRUE)
  saveRDS(x, path, version = 3)
  unname(tools::md5sum(path))
}

#' List the behavioral tests known to statwitness
#'
#' @return A data frame describing built-in validation tests, audit domains,
#'   and supported model families.
#' @export
witness_registry <- function() {
  ids <- c(
    "row_order", "formula_order", "model_frame", "prediction_reconstruction",
    "factor_label_invariance", "predictor_rescaling", "focus_reference_reversal",
    "outcome_translation", "outcome_scaling", "known_effect_injection",
    "independent_qr", "binary_outcome_reversal", "likelihood_reconstruction",
    "anova_cell_audit", "anova_term_order", "weight_semantics",
    "fixed_design_condition", "design_support", "model_convergence", "case_influence",
    "residual_shape", "residual_variance", "glm_dispersion",
    "binomial_separation", "group_label_invariance", "mixed_convergence",
    "mixed_singularity", "optimizer_agreement", "random_structure",
    "mixed_design_adequacy", "group_influence", "survival_time_scaling",
    "event_coding", "cox_ph_assumption", "survival_information",
    "survival_influence", "rm_design_integrity", "rm_row_order",
    "rm_id_label", "rm_outcome_translation", "rm_outcome_scaling",
    "rm_sphericity", "rm_residual_shape", "rm_id_influence"
  )
  models <- c(
    "all supported", "lm, aov, glm", "all supported", "all supported",
    "factor predictors", "simple numeric predictors", "two-level focus factors",
    "Gaussian lm/lmer/glmmTMB", "Gaussian lm/lmer/glmmTMB", "Gaussian lm/lmer",
    "lm/aov", "binomial glm", "binomial or Poisson glm",
    "factorial lm/aov", "factorial lm/aov", "weighted supported models",
    "all fixed-effect model matrices", "all supported", "glm/coxph/survreg and closed-form models",
    "lm/aov/glm", "Gaussian lm/aov/lmer", "Gaussian lm/aov/lmer",
    "glm/glmer/glmmTMB", "binomial glm/glmer/glmmTMB",
    "lmer/glmer/glmmTMB", "lmer/glmer/glmmTMB", "lmer/glmer/glmmTMB",
    "lmer/glmer", "lmer/glmer/glmmTMB", "lmer/glmer/glmmTMB",
    "lmer/glmer/glmmTMB", "coxph", "coxph/survreg", "coxph",
    "coxph/survreg", "coxph/survreg", "repeated-measures ANOVA",
    "repeated-measures ANOVA", "repeated-measures ANOVA",
    "repeated-measures ANOVA", "repeated-measures ANOVA",
    "repeated-measures ANOVA", "repeated-measures ANOVA",
    "repeated-measures ANOVA"
  )
  purpose <- c(
    "Checks invariance to row order.",
    "Checks invariance to equivalent ordering of fixed-effect terms.",
    "Reconstructs the analysis sample from supplied variables.",
    "Reconstructs fixed-effect predictions from the design matrix.",
    "Renames factor labels while preserving membership.",
    "Changes units of simple numeric predictors.",
    "Reverses a two-level reference category.",
    "Adds a constant to a Gaussian outcome.",
    "Rescales a Gaussian outcome.",
    "Injects a known linear effect.",
    "Recomputes ordinary linear coefficients by weighted QR.",
    "Reverses a 0/1 logit outcome.",
    "Reconstructs supported GLM likelihoods.",
    "Reports empty and unbalanced ANOVA cells.",
    "Checks sensitivity of an omnibus ANOVA term to term ordering.",
    "Checks that supplied and perturbed weights reach the fit.",
    "Checks fixed-effect rank and numerical conditioning.",
    "Reports basic observation, parameter, residual-df, and factor-level support.",
    "Inspects model-specific convergence indicators.",
    "Screens ordinary models with Cook's distance.",
    "Screens Gaussian residual asymmetry and tail heaviness.",
    "Screens residual-variance changes across fitted values.",
    "Checks Pearson dispersion for generalized models.",
    "Screens binomial models for numerical signs of separation.",
    "Renames random-effect grouping labels.",
    "Inspects mixed-model optimizer convergence diagnostics.",
    "Detects singular or boundary random-effect structures.",
    "Refits lme4 models with an alternative optimizer.",
    "Summarizes random-effect variance and correlation boundaries.",
    "Compares within-cluster predictors with registered random slopes.",
    "Refits after deleting one grouping level at a time.",
    "Changes Cox-model survival time units.",
    "Checks event-indicator coding.",
    "Runs Schoenfeld-residual proportional-hazards diagnostics.",
    "Reports event information relative to fitted model size.",
    "Screens survival DFBETA influence.",
    "Checks duplicate and missing subject-condition cells.",
    "Checks repeated-ANOVA invariance to row order.",
    "Renames participant identifiers.",
    "Adds a constant to the repeated outcome.",
    "Rescales the repeated outcome.",
    "Checks whether a sphericity correction is declared when relevant.",
    "Screens residual shape from the repeated-measures fit.",
    "Refits after deleting one participant at a time when feasible."
  )
  data.frame(
    id = ids,
    domain = vapply(ids, .test_domain, character(1)),
    models = models,
    purpose = purpose,
    stringsAsFactors = FALSE
  )
}

.build_plan <- function(info, audit_level = "standard") {
  audit_level <- match.arg(audit_level, c("core", "standard", "thorough"))
  selected <- list()
  skipped <- list()
  add_selected <- function(id, label, parameter = NA_character_) {
    selected[[length(selected) + 1L]] <<- data.frame(
      id = id, test = label, parameter = parameter, status = "SELECTED",
      domain = .test_domain(id), reason = "Applicable to this analysis.",
      stringsAsFactors = FALSE)
  }
  add_skipped <- function(id, label, reason) {
    skipped[[length(skipped) + 1L]] <<- data.frame(
      id = id, test = label, parameter = NA_character_, status = "SKIPPED",
      domain = .test_domain(id), reason = reason, stringsAsFactors = FALSE)
  }

  # Shared integrity and numerical domains.
  add_selected("row_order", "Row-order invariance")
  add_selected("model_frame", "Analysis-sample reconstruction")
  add_selected("prediction_reconstruction", "Fixed-prediction reconstruction")
  add_selected("fixed_design_condition", "Fixed-effect design conditioning")
  add_selected("design_support", "Basic design-support audit")

  if (!info$is_mixed && !info$is_survival && length(info$term_labels) > 1L) {
    add_selected("formula_order", "Formula-order invariance")
  } else {
    add_skipped("formula_order", "Formula-order invariance", "Formula reordering is limited to non-mixed, non-survival fixed-effect formulas.")
  }

  if (info$weights_supplied) add_selected("weight_semantics", "Weight-semantics validation") else
    add_skipped("weight_semantics", "Weight-semantics validation", "No analysis weights were supplied.")

  if (length(info$factor_predictors)) {
    for (v in info$factor_predictors) add_selected("factor_label_invariance", paste0("Factor-label invariance: ", v), v)
  } else add_skipped("factor_label_invariance", "Factor-label invariance", "No simple factor predictors were found.")

  eligible_numeric <- intersect(info$numeric_predictors, info$simple_only_predictors)
  if (length(eligible_numeric)) {
    for (v in eligible_numeric) add_selected("predictor_rescaling", paste0("Predictor-unit rescaling: ", v), v)
  } else add_skipped("predictor_rescaling", "Predictor-unit rescaling", "No simple numeric predictor outside an interaction was found.")

  if (info$focus_kind == "coefficient" && info$focus_binary_factor && !info$focus_in_interaction) {
    add_selected("focus_reference_reversal", "Focus-reference reversal", info$focus)
  } else add_skipped("focus_reference_reversal", "Focus-reference reversal", "Requires a coefficient-level two-level factor focus outside an interaction.")

  gaussian_models <- c("lm", "aov", "lmerMod", "glmmTMB")
  is_gaussian <- info$model_class %in% gaussian_models && .is_gaussian_identity(info)
  if (is_gaussian && info$response_simple && info$response_name %in% names(info$data) && is.numeric(info$data[[info$response_name]])) {
    if (info$has_intercept) add_selected("outcome_translation", "Outcome translation") else
      add_skipped("outcome_translation", "Outcome translation", "Requires an intercept.")
    add_selected("outcome_scaling", "Outcome-unit scaling")
    if (info$focus_kind == "coefficient" && !info$focus_in_interaction &&
        (info$focus %in% eligible_numeric || info$focus_binary_factor) && info$model_class != "glmmTMB") {
      add_selected("known_effect_injection", "Known-effect injection", info$focus)
    } else add_skipped("known_effect_injection", "Known-effect injection", "Requires a simple coefficient-level focus in a supported Gaussian model.")
  } else {
    add_skipped("outcome_translation", "Outcome translation", "Only defined for direct Gaussian identity-link outcomes.")
    add_skipped("outcome_scaling", "Outcome-unit scaling", "Only defined for direct Gaussian identity-link outcomes.")
    add_skipped("known_effect_injection", "Known-effect injection", "Only defined for supported Gaussian models with a simple focus.")
  }

  if (info$model_class %in% c("lm", "aov")) add_selected("independent_qr", "Independent weighted-QR coefficient calculation") else
    add_skipped("independent_qr", "Independent weighted-QR coefficient calculation", "Only applies to ordinary linear models.")

  # Ordinary linear/ANOVA domains.
  if (info$model_class %in% c("lm", "aov")) {
    add_selected("model_convergence", "Model convergence audit")
    if (audit_level != "core") {
      add_selected("case_influence", "Case-level influence screening")
      add_selected("residual_shape", "Residual-shape screening")
      add_selected("residual_variance", "Residual-variance screening")
    }
  }

  # GLM domains.
  if (info$model_class == "glm") {
    add_selected("model_convergence", "Model convergence audit")
    y <- stats::model.response(stats::model.frame(info$model))
    if (info$family == "binomial" && info$link == "logit" && is.numeric(y) && all(y %in% c(0, 1))) {
      add_selected("binary_outcome_reversal", "Binary-outcome reversal")
    } else add_skipped("binary_outcome_reversal", "Binary-outcome reversal", "Requires a Bernoulli 0/1 outcome with a logit link.")
    if ((info$family == "binomial" && is.numeric(y) && all(y %in% c(0, 1))) || info$family == "poisson") {
      add_selected("likelihood_reconstruction", "Independent log-likelihood reconstruction")
    } else add_skipped("likelihood_reconstruction", "Independent log-likelihood reconstruction", "Supports Bernoulli-binomial and Poisson GLMs.")
    if (audit_level != "core") {
      add_selected("glm_dispersion", "Generalized-model dispersion audit")
      add_selected("case_influence", "Case-level influence screening")
      if (info$family == "binomial" && info$link == "logit") {
        add_selected("binomial_separation", "Binomial separation screening")
      }
    }
  } else {
    add_skipped("binary_outcome_reversal", "Binary-outcome reversal", "Currently implemented for ordinary GLMs.")
    add_skipped("likelihood_reconstruction", "Independent log-likelihood reconstruction", "Currently implemented for ordinary GLMs.")
  }

  # Factorial ANOVA domains.
  if (info$model_class %in% c("lm", "aov") && info$all_factor_design && length(info$factor_predictors) >= 2L) {
    add_selected("anova_cell_audit", "ANOVA cell-structure audit")
    add_selected("anova_term_order", "ANOVA term-order sensitivity")
  } else {
    add_skipped("anova_cell_audit", "ANOVA cell-structure audit", "Requires at least two simple factor predictors in an ordinary linear model.")
    add_skipped("anova_term_order", "ANOVA term-order sensitivity", "Requires a factorial ordinary linear model.")
  }

  # Mixed models get parallel numerical, design, assumption, and influence domains.
  if (info$is_mixed) {
    for (v in info$random_groups) add_selected("group_label_invariance", paste0("Grouping-label invariance: ", v), v)
    add_selected("mixed_convergence", "Mixed-model convergence audit")
    add_selected("mixed_singularity", "Random-effects singularity audit")
    add_selected("random_structure", "Random-effects structure summary")
    add_selected("mixed_design_adequacy", "Random-effects design audit")
    if (info$model_class %in% c("lmerMod", "glmerMod")) add_selected("optimizer_agreement", "Alternative-optimizer agreement") else
      add_skipped("optimizer_agreement", "Alternative-optimizer agreement", "Currently implemented for lme4 models.")
    if (audit_level != "core") {
      for (v in info$random_groups) add_selected("group_influence", paste0("Group-deletion influence: ", v), v)
      if (is_gaussian) {
        add_selected("residual_shape", "Residual-shape screening")
        add_selected("residual_variance", "Residual-variance screening")
      } else {
        add_selected("glm_dispersion", "Generalized-model dispersion audit")
        if (info$family == "binomial" && info$link == "logit") add_selected("binomial_separation", "Binomial separation screening")
      }
    }
  } else {
    add_skipped("group_label_invariance", "Grouping-label invariance", "No random-effect grouping factor was detected.")
    add_skipped("mixed_convergence", "Mixed-model convergence audit", "Not a mixed-effects model.")
    add_skipped("mixed_singularity", "Random-effects singularity audit", "Not a mixed-effects model.")
    add_skipped("random_structure", "Random-effects structure summary", "Not a mixed-effects model.")
    add_skipped("mixed_design_adequacy", "Random-effects design audit", "Not a mixed-effects model.")
    add_skipped("optimizer_agreement", "Alternative-optimizer agreement", "Not an lme4 mixed-effects model.")
    add_skipped("group_influence", "Group-deletion influence", "Not a mixed-effects model.")
  }

  # Survival domains.
  if (info$is_survival) {
    add_selected("model_convergence", "Model convergence audit")
    if (info$model_class == "coxph") add_selected("survival_time_scaling", "Survival-time unit scaling") else
      add_skipped("survival_time_scaling", "Survival-time unit scaling", "Automatic time-unit invariance is currently certified for Cox models only.")
    add_selected("event_coding", "Event-coding audit")
    add_selected("survival_information", "Survival information audit")
    if (info$model_class == "coxph") add_selected("cox_ph_assumption", "Proportional-hazards diagnostic") else
      add_skipped("cox_ph_assumption", "Proportional-hazards diagnostic", "Only applies to Cox models.")
    if (audit_level != "core") add_selected("survival_influence", "Survival influence screening")
  } else {
    add_skipped("survival_time_scaling", "Survival-time unit scaling", "Not a survival model.")
    add_skipped("event_coding", "Event-coding audit", "Not a survival model.")
    add_skipped("cox_ph_assumption", "Proportional-hazards diagnostic", "Not a Cox model.")
    add_skipped("survival_information", "Survival information audit", "Not a survival model.")
    add_skipped("survival_influence", "Survival influence screening", "Not a survival model.")
  }

  selected_df <- if (length(selected)) do.call(rbind, selected) else data.frame()
  skipped_df <- if (length(skipped)) do.call(rbind, skipped) else data.frame()
  list(selected = selected_df, skipped = skipped_df)
}

#' Preview the validation tests selected for an analysis
#'
#' @inheritParams statwitness
#' @return A data frame containing selected and skipped tests with reasons.
#' @export
audit_plan <- function(formula, data, focus = NULL, family = NULL, weights = NULL,
                       method = "auto", engine = NULL, control = NULL,
                       audit_level = c("standard", "core", "thorough")) {
  audit_level <- match.arg(audit_level)
  focus_expr <- substitute(focus)
  focus_value <- if (missing(focus) || identical(focus_expr, quote(NULL))) NULL else if (is.symbol(focus_expr) && as.character(focus_expr) %in% names(data)) as.character(focus_expr) else focus
  weights_expr <- substitute(weights)
  weights_vec <- if (missing(weights)) NULL else if (is.symbol(weights_expr) && as.character(weights_expr) %in% names(data)) data[[as.character(weights_expr)]] else .resolve_weights(data, weights, weights_expr, FALSE)
  resolved_method <- .detect_method(formula, family, method)
  model <- .fit_analysis(formula, data, family, weights_vec, resolved_method, engine, control)
  info <- .inspect_analysis(model, formula, data, focus_value, weights_vec, engine, resolved_method, control)
  plan <- .build_plan(info, audit_level = audit_level)
  out <- rbind(plan$selected, plan$skipped)
  rownames(out) <- NULL
  out
}

test_that("audit plan explains selected and skipped tests", {
  plan <- audit_plan(mpg ~ wt + hp, data = mtcars, focus = "wt")
  expect_s3_class(plan, "data.frame")
  expect_true(all(c("id", "status", "reason") %in% names(plan)))
  expect_true(any(plan$status == "SELECTED"))
  expect_true(any(plan$status == "SKIPPED"))
})

test_that("registry is available", {
  reg <- witness_registry()
  expect_s3_class(reg, "data.frame")
  expect_true(nrow(reg) >= 10L)
  expect_true(all(c("id", "models", "purpose") %in% names(reg)))
})


test_that("bare focus variable is supported", {
  dat <- transform(mtcars, transmission = factor(am))
  out <- statwitness(mpg ~ transmission + wt, data = dat, focus = transmission)
  expect_identical(out$focus, "transmission")
})

test_that("standard audits return the five cross-model domains where applicable", {
  dat <- transform(mtcars, transmission = factor(am))
  out <- statwitness(
    mpg ~ transmission + wt + hp,
    data = dat,
    focus = "transmission",
    audit_level = "standard"
  )

  expect_true(all(c("domain", "status") %in% names(out$domains)))
  expect_true(all(c(
    "computational_integrity", "numerical_stability",
    "assumption_screening", "influence_stability"
  ) %in% out$domains$domain))
  expect_true(any(out$tests$id == "fixed_design_condition"))
  expect_true(any(out$tests$id == "case_influence"))
  expect_true(any(out$tests$id == "residual_shape"))
})

test_that("core level omits assumption and influence screens", {
  out <- statwitness(mpg ~ wt + hp, data = mtcars, focus = "wt", audit_level = "core")
  expect_false(any(out$tests$id %in% c("case_influence", "residual_shape", "residual_variance")))
  expect_true(any(out$tests$id == "fixed_design_condition"))
})

test_that("registry exposes domains for every registered test", {
  reg <- witness_registry()
  expect_true(all(nzchar(reg$domain)))
  expect_identical(anyDuplicated(reg$id), 0L)
})

#' statwitness: Model-Aware Validation and Audit Certificates
#'
#' Model-aware computational validation for regression, ANOVA,
#' repeated-measures, mixed-effects, and survival workflows.
#' The certificates concern registered computational behavior and selected
#' diagnostics; they do not certify causal or substantive validity.
#'
#' @keywords internal
"_PACKAGE"

.infer_focus <- function(model, fixed_formula, data, focus = NULL) {
  coefs <- .fixed_coef(model)
  coef_names <- names(coefs)
  term_labels <- attr(stats::terms(fixed_formula), "term.labels")
  candidates <- coef_names[coef_names != "(Intercept)" & !is.na(coefs)]

  if (is.null(focus)) {
    if (!length(candidates)) .sw_abort("The fitted model has no non-intercept coefficient to audit.")
    return(list(variable = candidates[[1L]], coefficient = candidates[[1L]],
                term = NA_character_, kind = "coefficient", inferred = TRUE))
  }

  focus <- as.character(focus)[[1L]]
  if (focus %in% coef_names) {
    return(list(variable = focus, coefficient = focus, term = NA_character_,
                kind = "coefficient", inferred = FALSE))
  }
  if (focus %in% names(data)) {
    matches <- coef_names[startsWith(coef_names, focus)]
    matches <- matches[!grepl(":", matches, fixed = TRUE)]
    matches <- matches[!is.na(coefs[matches])]
    if (length(matches) == 1L) {
      return(list(variable = focus, coefficient = matches[[1L]], term = NA_character_,
                  kind = "coefficient", inferred = FALSE))
    }
    if (length(matches) > 1L) {
      return(list(variable = focus, coefficient = NA_character_, term = focus,
                  kind = "term", inferred = FALSE))
    }
  }
  if (focus %in% term_labels) {
    return(list(variable = focus, coefficient = NA_character_, term = focus,
                kind = "term", inferred = FALSE))
  }

  .sw_abort(
    "Could not match `focus`. Available terms are: ", paste(term_labels, collapse = ", "),
    ". Available coefficients are: ", paste(coef_names, collapse = ", "), "."
  )
}

.inspect_analysis <- function(model, formula, data, focus, weights, engine,
                              method, control) {
  fixed_formula <- .fixed_formula(formula)
  trm <- stats::terms(fixed_formula)
  labels <- attr(trm, "term.labels")
  response_expr <- fixed_formula[[2L]]
  response_simple <- is.symbol(response_expr)
  response_name <- if (response_simple) as.character(response_expr) else NA_character_
  all_predictors <- all.vars(stats::delete.response(trm))
  simple_predictors <- intersect(labels, names(data))
  numeric_predictors <- simple_predictors[vapply(data[simple_predictors], function(x) {
    is.numeric(x) || is.integer(x)
  }, logical(1))]
  factor_predictors <- simple_predictors[vapply(data[simple_predictors], function(x) {
    is.factor(x) || is.character(x) || is.logical(x)
  }, logical(1))]
  simple_only_predictors <- simple_predictors[vapply(simple_predictors, function(v) {
    uses <- vapply(labels, function(lbl) {
      v %in% all.vars(stats::as.formula(paste("~", lbl)))
    }, logical(1))
    sum(uses) == 1L && identical(labels[uses], v)
  }, logical(1))]

  focus_info <- .infer_focus(model, fixed_formula, data, focus)
  focus_var <- focus_info$variable
  focus_in_interaction <- any(grepl(":", labels, fixed = TRUE) &
    vapply(strsplit(labels, ":", fixed = TRUE), function(x) focus_var %in% x, logical(1)))
  focus_binary_factor <- focus_var %in% names(data) &&
    (is.factor(data[[focus_var]]) || is.character(data[[focus_var]]) || is.logical(data[[focus_var]])) &&
    length(unique(stats::na.omit(data[[focus_var]]))) == 2L

  id <- .model_class_id(model)
  fam <- .model_family_info(model)
  all_factor_design <- length(simple_predictors) >= 1L &&
    length(simple_predictors) == length(factor_predictors)
  is_mixed <- id %in% c("lmerMod", "glmerMod", "glmmTMB")
  is_survival <- id %in% c("coxph", "survreg")
  model_type <- switch(
    id,
    lm = if (all_factor_design) "anova_linear_model" else "linear_model",
    aov = "classical_anova",
    glm = paste0(fam$family, "_glm"),
    lmerMod = "linear_mixed_model",
    glmerMod = paste0(fam$family, "_generalized_mixed_model"),
    glmmTMB = paste0(fam$family, "_glmmTMB_model"),
    coxph = "cox_proportional_hazards",
    survreg = "parametric_survival_model",
    id
  )

  list(
    model = model, formula = formula, fixed_formula = fixed_formula, data = data,
    weights = weights, weights_supplied = !is.null(weights),
    engine = engine %||% .default_engine, method = method, control = control,
    family_object = fam$object, model_class = id, model_type = model_type,
    family = fam$family, link = fam$link, terms = trm, term_labels = labels,
    has_intercept = attr(trm, "intercept") == 1L,
    response_expr = response_expr, response_simple = response_simple,
    response_name = response_name, response_variables = .response_variables(formula),
    all_predictors = all_predictors, simple_predictors = simple_predictors,
    simple_only_predictors = simple_only_predictors,
    numeric_predictors = numeric_predictors, factor_predictors = factor_predictors,
    has_interactions = any(grepl(":", labels, fixed = TRUE)),
    focus = focus_var, focus_coef = focus_info$coefficient,
    focus_term = focus_info$term, focus_kind = focus_info$kind,
    focus_inferred = focus_info$inferred,
    focus_in_interaction = focus_in_interaction,
    focus_binary_factor = focus_binary_factor,
    all_factor_design = all_factor_design,
    is_mixed = is_mixed, is_survival = is_survival,
    random_groups = .random_group_vars(formula)
  )
}

.term_lrt <- function(info, term) {
  reduced_formula <- stats::update.formula(info$formula, paste(". ~ . -", term))
  full <- info$model
  if (info$model_class == "lmerMod") {
    full_args <- list(formula = info$formula, data = info$data,
                      REML = FALSE, na.action = stats::na.omit)
    reduced_args <- list(formula = reduced_formula, data = info$data,
                         REML = FALSE, na.action = stats::na.omit)
    if (!is.null(info$weights)) {
      full_args$weights <- info$weights
      reduced_args$weights <- info$weights
    }
    full <- do.call(lme4::lmer, full_args)
    reduced <- do.call(lme4::lmer, reduced_args)
  } else {
    reduced <- .refit(info, info$data, formula = reduced_formula)
  }
  tab <- stats::anova(reduced, full)
  pcol <- grep("Pr\\(", colnames(tab), value = TRUE)
  statcol <- intersect(c("Chisq", "L.Ratio", "F"), colnames(tab))
  dfcol <- intersect(c("Chi Df", "Df"), colnames(tab))
  list(
    statistic = if (length(statcol)) unname(tab[nrow(tab), statcol[[1L]]]) else NA_real_,
    df = if (length(dfcol)) unname(tab[nrow(tab), dfcol[[1L]]]) else NA_real_,
    p_value = if (length(pcol)) unname(tab[nrow(tab), pcol[[1L]]]) else NA_real_,
    method = "full-versus-reduced likelihood-ratio test"
  )
}

.term_test_lm <- function(info, term) {
  test <- if (info$model_class %in% c("lm", "aov")) "F" else "Chisq"
  tab <- tryCatch(stats::drop1(info$model, test = test), error = function(e) NULL)
  if (is.null(tab) || !term %in% rownames(tab)) {
    tab <- if (info$model_class == "glm") stats::anova(info$model, test = "Chisq") else stats::anova(info$model)
  }
  rn <- rownames(tab)
  idx <- which(rn == term)
  if (!length(idx)) .sw_abort("The omnibus term `", term, "` could not be isolated under the model hierarchy.")
  pcol <- grep("Pr\\(", colnames(tab), value = TRUE)
  statcol <- intersect(c("F value", "LRT"), colnames(tab))
  dfcol <- intersect(c("Df"), colnames(tab))
  list(
    statistic = if (length(statcol)) unname(tab[idx[[1L]], statcol[[1L]]]) else NA_real_,
    df = if (length(dfcol)) unname(tab[idx[[1L]], dfcol[[1L]]]) else NA_real_,
    p_value = if (length(pcol)) unname(tab[idx[[1L]], pcol[[1L]]]) else NA_real_,
    method = paste0("drop-one ", test, " test")
  )
}

.primary_result <- function(info, conf_level = 0.95) {
  if (info$focus_kind == "term") {
    tt <- if (info$model_class %in% c("lm", "aov", "glm")) {
      .term_test_lm(info, info$focus_term)
    } else {
      .term_lrt(info, info$focus_term)
    }
    return(list(
      kind = "term", label = info$focus_term, coefficient = NA_character_,
      estimate = NA_real_, standard_error = NA_real_, statistic = tt$statistic,
      df = tt$df, p_value = tt$p_value, conf_level = conf_level,
      lower = NA_real_, upper = NA_real_, transformed = NULL,
      n = .model_nobs(info$model), method = tt$method
    ))
  }

  beta <- .fixed_coef(info$model)
  V <- .fixed_vcov(info$model)
  if (!info$focus_coef %in% names(beta)) {
    .sw_abort("The focus coefficient is not estimable in the fitted model.")
  }
  estimate <- unname(beta[[info$focus_coef]])
  se <- sqrt(unname(V[info$focus_coef, info$focus_coef]))
  statistic <- estimate / se
  pvals <- .model_p_values(info$model)
  p_value <- if (info$focus_coef %in% names(pvals)) pvals[[info$focus_coef]] else 2 * stats::pnorm(abs(statistic), lower.tail = FALSE)
  alpha <- 1 - conf_level
  critical <- if (info$model_class %in% c("lm", "aov")) {
    stats::qt(1 - alpha / 2, df = stats::df.residual(info$model))
  } else {
    stats::qnorm(1 - alpha / 2)
  }
  lower <- estimate - critical * se
  upper <- estimate + critical * se
  transformed <- NULL
  if (info$link == "logit") {
    transformed <- list(label = "odds ratio", estimate = exp(estimate), lower = exp(lower), upper = exp(upper))
  } else if (info$link %in% c("log", "log-hazard")) {
    transformed <- list(label = if (info$link == "log-hazard") "hazard ratio" else "rate ratio",
                        estimate = exp(estimate), lower = exp(lower), upper = exp(upper))
  }
  list(
    kind = "coefficient", label = info$focus_coef, coefficient = info$focus_coef,
    estimate = estimate, standard_error = se, statistic = statistic, df = NA_real_,
    p_value = p_value, conf_level = conf_level, lower = lower, upper = upper,
    transformed = transformed, n = .model_nobs(info$model), method = "coefficient-level Wald inference"
  )
}

.test_binary_outcome_reversal <- function(info, tolerance) {
  d2 <- info$data
  yname <- info$response_name
  if (!info$response_simple || is.na(yname)) {
    return(.new_test(
      "binary_outcome_reversal", "Binary-outcome reversal", "ERROR",
      expected = "A simple 0/1 response variable.",
      observed = "The response is transformed or unavailable.",
      details = "Version 0.1 supports direct binary response columns only."
    ))
  }
  d2[[yname]] <- 1 - d2[[yname]]
  m2 <- .refit(info, d2)
  c1 <- stats::coef(info$model)
  c2 <- stats::coef(m2)
  ov <- .named_overlap(-c1, c2)
  p1 <- stats::fitted(info$model)
  p2 <- stats::fitted(m2)
  ok <- .within_tol(ov$x, ov$y, tolerance * 10) && .within_tol(p1, 1 - p2, tolerance * 10)
  .new_test(
    "binary_outcome_reversal", "Binary-outcome reversal", if (ok) "PASS" else "FAIL",
    expected = "All coefficients reverse sign and original probabilities equal one minus recoded probabilities.",
    observed = paste0(
      "maximum coefficient difference from sign reversal = ", .format_num(.max_abs_diff(ov$x, ov$y)),
      "; maximum probability identity error = ", .format_num(.max_abs_diff(p1, 1 - p2))
    ),
    details = if (ok) "Binary outcome reversal behaved as required by the logit model." else "The recoded binary model did not show the expected symmetry."
  )
}

.test_likelihood <- function(info, tolerance) {
  model <- info$model
  y <- stats::model.response(stats::model.frame(model))
  mu <- stats::fitted(model)
  w <- model$prior.weights %||% rep(1, length(y))
  reconstructed <- if (info$family == "binomial") {
    m <- w
    contribution_weight <- ifelse(m > 0, w / m, 0)
    sum(contribution_weight * stats::dbinom(
      round(m * y), size = round(m), prob = mu, log = TRUE
    ))
  } else if (info$family == "poisson") {
    sum(w * stats::dpois(y, lambda = mu, log = TRUE))
  } else {
    NA_real_
  }
  reported <- as.numeric(stats::logLik(model))
  ok <- .within_tol(reconstructed, reported, tolerance * 100)
  .new_test(
    "likelihood_reconstruction", "Independent log-likelihood reconstruction", if (ok) "PASS" else "FAIL",
    expected = "The reported likelihood matches a direct probability-mass calculation.",
    observed = paste0(
      "reported = ", .format_num(reported), "; reconstructed = ", .format_num(reconstructed),
      "; absolute difference = ", .format_num(abs(reported - reconstructed))
    ),
    details = if (ok) "The model likelihood was independently reconstructed." else "The reported likelihood differs from the reconstructed value."
  )
}

# Your model: random intercept for rater and an omnibus interaction audit.
# Requires install.packages("lme4").

audit <- statwitness::statwitness(
  score ~ criterion * model + (1 | rater),
  data = dataset,
  focus = "criterion:model"
)

print(audit, details = TRUE)

# Repeated-measures ANOVA alternative for a complete long-format design.
# Requires install.packages("afex").

rm_audit <- statwitness::statwitness_repeated(
  data = dataset,
  outcome = "score",
  id = "rater",
  within = c("criterion", "model"),
  focus = "criterion:model"
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## -----------------------------------------------------------------------------
library(statwitness)
dat <- transform(mtcars, transmission = factor(am))
audit <- statwitness(mpg ~ transmission + wt + hp, data = dat, focus = "transmission")
audit

## ----eval = requireNamespace("lme4", quietly = TRUE)--------------------------
dat <- lme4::sleepstudy
dat$criterion <- factor(rep(c("A", "B"), length.out = nrow(dat)))
dat$model <- factor(rep(c("M1", "M2"), each = 2, length.out = nrow(dat)))
dat$rater <- dat$Subject

audit <- statwitness(
  Reaction ~ criterion * model + (1 | rater),
  data = dat,
  focus = "criterion:model",
  exclude = "optimizer_agreement",
  audit_level = "core"
)
audit

## ----eval = FALSE-------------------------------------------------------------
# statwitness_repeated(
#   data = dataset,
#   outcome = "score",
#   id = "rater",
#   within = c("criterion", "model"),
#   focus = "criterion:model",
#   audit_level = "standard"
# )

.test_row_order <- function(info, tolerance, seed) {
  set.seed(seed)
  ord <- sample.int(nrow(info$data))
  d2 <- info$data[ord, , drop = FALSE]
  w2 <- if (is.null(info$weights)) NULL else info$weights[ord]
  m2 <- .refit(info, d2, weights = w2)
  ov <- .named_overlap(.fixed_coef(info$model), .fixed_coef(m2))
  ll1 <- .model_loglik(info$model)
  ll2 <- .model_loglik(m2)
  ok_coef <- .within_tol(ov$x, ov$y, tolerance * 100)
  ok_ll <- if (is.na(ll1) || is.na(ll2)) TRUE else .within_tol(ll1, ll2, tolerance * 100)
  ok <- ok_coef && ok_ll
  .new_test(
    "row_order", "Row-order invariance", if (ok) "PASS" else "FAIL",
    expected = "Equivalent fixed effects and likelihood after permuting rows.",
    observed = paste0("maximum fixed-effect difference = ", .format_num(.max_abs_diff(ov$x, ov$y)),
                      "; likelihood difference = ", .format_num(abs(ll1 - ll2))),
    details = if (ok) "The analysis was invariant to row order." else "The result changed after row permutation."
  )
}

.reordered_formula <- function(formula) {
  trm <- stats::terms(formula)
  labels <- rev(attr(trm, "term.labels"))
  response <- paste(deparse(formula[[2L]]), collapse = "")
  f2 <- stats::reformulate(labels, response = response, intercept = attr(trm, "intercept"))
  environment(f2) <- environment(formula)
  f2
}

.test_formula_order <- function(info, tolerance) {
  f2 <- .reordered_formula(info$formula)
  m2 <- .refit(info, info$data, formula = f2)
  ov <- .named_overlap(.fixed_coef(info$model), .fixed_coef(m2))
  p1 <- .model_fixed_prediction(info$model)
  p2 <- .model_fixed_prediction(m2)
  ok <- .within_tol(ov$x, ov$y, tolerance * 10) && .within_tol(p1, p2, tolerance * 10)
  .new_test(
    "formula_order", "Formula-order invariance", if (ok) "PASS" else "FAIL",
    expected = "Equivalent coefficients and predictions after reordering formula terms.",
    observed = paste0("maximum coefficient difference = ", .format_num(.max_abs_diff(ov$x, ov$y)),
                      "; maximum prediction difference = ", .format_num(.max_abs_diff(p1, p2))),
    details = if (ok) "Equivalent term order produced the same fitted analysis." else "Equivalent formula ordering changed the fitted result."
  )
}

.test_model_frame <- function(info) {
  vars <- intersect(.analysis_variables(info$formula), names(info$data))
  keep <- stats::complete.cases(info$data[vars])
  if (!is.null(info$weights)) keep <- keep & !is.na(info$weights)
  expected_n <- sum(keep)
  observed_n <- .model_nobs(info$model)
  mf <- .model_frame_safe(info$model)
  same_rows <- NA
  if (!is.null(mf)) {
    expected_rows <- rownames(info$data)[keep]
    observed_rows <- rownames(mf)
    same_rows <- identical(sort(expected_rows), sort(observed_rows))
  }
  ok <- expected_n == observed_n && (is.na(same_rows) || same_rows)
  .new_test(
    "model_frame", "Analysis-sample reconstruction", if (ok) "PASS" else "FAIL",
    expected = paste0(expected_n, " complete analysis rows reconstructed from the supplied data."),
    observed = paste0(observed_n, " rows in the fitted model; row identities matched = ", same_rows, "."),
    details = if (ok) "The fitted sample size and available row identities were reconstructed." else "The fitted sample did not match the supplied data and formula."
  )
}

.test_prediction_reconstruction <- function(info, tolerance) {
  model <- info$model
  X <- .model_matrix_fixed(model)
  beta <- .fixed_coef(model)
  estimable <- !is.na(beta)
  common <- intersect(colnames(X), names(beta)[estimable])
  eta <- drop(X[, common, drop = FALSE] %*% beta[common])
  mf <- .model_frame_safe(model)
  if (!is.null(mf)) {
    off <- stats::model.offset(mf)
    if (!is.null(off)) eta <- eta + off
  }
  observed <- .model_fixed_prediction(model)
  if (info$model_class == "coxph") {
    eta <- eta - mean(eta)
    observed <- observed - mean(observed)
  }
  ok <- .within_tol(eta, observed, tolerance * 100)
  .new_test(
    "prediction_reconstruction", "Fixed-prediction reconstruction", if (ok) "PASS" else "FAIL",
    expected = "Fixed-effect predictions reconstructed from the model matrix and coefficients.",
    observed = paste0("maximum reconstruction difference = ", .format_num(.max_abs_diff(eta, observed))),
    details = if (ok) "The fixed-effect linear predictor was independently reconstructed." else "Stored predictions disagree with the reconstructed fixed-effect calculation."
  )
}

.test_predictor_rescaling <- function(info, variable, tolerance) {
  d2 <- info$data
  d2[[variable]] <- d2[[variable]] * 10
  m2 <- .refit(info, d2)
  c1 <- .fixed_coef(info$model)
  c2 <- .fixed_coef(m2)
  expected <- c1
  if (!variable %in% names(c1)) {
    return(.new_test("predictor_rescaling", paste0("Predictor-unit rescaling: ", variable), "ERROR",
      expected = "A direct coefficient for the rescaled predictor.",
      observed = "No coefficient with the predictor name was found.",
      details = "This test supports simple numeric main effects only."))
  }
  expected[[variable]] <- expected[[variable]] / 10
  ov <- .named_overlap(expected, c2)
  p1 <- .model_fixed_prediction(info$model)
  p2 <- .model_fixed_prediction(m2)
  ok <- .within_tol(ov$x, ov$y, tolerance * 100) && .within_tol(p1, p2, tolerance * 100)
  .new_test(
    "predictor_rescaling", paste0("Predictor-unit rescaling: ", variable), if (ok) "PASS" else "FAIL",
    expected = paste0("The ", variable, " coefficient divides by 10 while fixed predictions remain unchanged."),
    observed = paste0("coefficient difference from expectation = ", .format_num(.max_abs_diff(ov$x, ov$y)),
                      "; maximum prediction difference = ", .format_num(.max_abs_diff(p1, p2))),
    details = if (ok) "Changing predictor units propagated correctly." else "Changing predictor units did not propagate as expected."
  )
}

.test_focus_reference <- function(info, tolerance) {
  v <- info$focus
  d2 <- info$data
  d2[[v]] <- factor(d2[[v]])
  old_levels <- levels(d2[[v]])
  d2[[v]] <- stats::relevel(d2[[v]], ref = old_levels[[2L]])
  m2 <- .refit(info, d2)
  c1 <- .fixed_coef(info$model)[[info$focus_coef]]
  candidates <- names(.fixed_coef(m2))[startsWith(names(.fixed_coef(m2)), v)]
  candidates <- candidates[!grepl(":", candidates, fixed = TRUE)]
  candidates <- candidates[!is.na(.fixed_coef(m2)[candidates])]
  if (length(candidates) != 1L) {
    return(.new_test("focus_reference_reversal", "Focus-reference reversal", "ERROR",
      expected = "Exactly one reversed focus coefficient.",
      observed = paste(length(candidates), "candidate coefficients found."),
      details = "The factor parameterization could not be matched automatically."))
  }
  c2 <- .fixed_coef(m2)[[candidates[[1L]]]]
  p1 <- .model_fixed_prediction(info$model)
  p2 <- .model_fixed_prediction(m2)
  ok <- .within_tol(c2, -c1, tolerance * 100) && .within_tol(p1, p2, tolerance * 100)
  .new_test(
    "focus_reference_reversal", "Focus-reference reversal", if (ok) "PASS" else "FAIL",
    expected = "The focus coefficient reverses sign while fixed predictions remain unchanged.",
    observed = paste0("original = ", .format_num(c1), "; reversed = ", .format_num(c2),
                      "; maximum prediction difference = ", .format_num(.max_abs_diff(p1, p2))),
    details = if (ok) "Reference-category reversal behaved as expected." else "Reference-category reversal changed more than the parameterization."
  )
}

.test_factor_label_invariance <- function(info, variable, tolerance) {
  d2 <- info$data
  x <- factor(d2[[variable]])
  levels(x) <- paste0("SW_LEVEL_", seq_along(levels(x)))
  d2[[variable]] <- x
  m2 <- .refit(info, d2)
  p1 <- .model_fixed_prediction(info$model)
  p2 <- .model_fixed_prediction(m2)
  ll1 <- .model_loglik(info$model)
  ll2 <- .model_loglik(m2)
  ok <- .within_tol(p1, p2, tolerance * 100) && (is.na(ll1) || .within_tol(ll1, ll2, tolerance * 100))
  .new_test(
    "factor_label_invariance", paste0("Factor-label invariance: ", variable), if (ok) "PASS" else "FAIL",
    expected = "Renaming factor labels without changing membership preserves predictions and likelihood.",
    observed = paste0("maximum prediction difference = ", .format_num(.max_abs_diff(p1, p2)),
                      "; likelihood difference = ", .format_num(abs(ll1 - ll2))),
    details = if (ok) "Factor labels were treated as labels rather than data values." else "Renaming factor labels changed the fitted analysis."
  )
}

.effective_model_weights <- function(model) {
  if (inherits(model, "glm")) return(model$prior.weights)
  if (inherits(model, "lm")) return(model$weights)
  if (inherits(model, "merMod")) return(tryCatch(stats::weights(model), error = function(e) NULL))
  if (inherits(model, "coxph")) return(model$weights)
  NULL
}

.test_weight_semantics <- function(info, tolerance) {
  vars <- intersect(.analysis_variables(info$formula), names(info$data))
  keep <- stats::complete.cases(info$data[vars]) & !is.na(info$weights)
  expected_original <- info$weights[keep]
  observed_original <- .effective_model_weights(info$model)
  if (is.null(observed_original)) {
    return(.new_test("weight_semantics", "Weight-semantics validation", "FAIL",
      expected = "The fitted model records the supplied analysis weights.",
      observed = "The fitted model contains no accessible effective weights.",
      details = "The fitting engine may have ignored the supplied weights."))
  }
  perturbed <- info$weights
  idx <- which(!is.na(perturbed))
  perturbed[idx] <- perturbed[idx] * rep(c(1, 2), length.out = length(idx))
  m2 <- .refit(info, info$data, weights = perturbed)
  observed_perturbed <- .effective_model_weights(m2)
  expected_perturbed <- perturbed[keep]
  original_ok <- .within_tol(expected_original, observed_original, tolerance * 100)
  perturbed_ok <- !is.null(observed_perturbed) && .within_tol(expected_perturbed, observed_perturbed, tolerance * 100)
  ok <- original_ok && perturbed_ok
  .new_test(
    "weight_semantics", "Weight-semantics validation", if (ok) "PASS" else "FAIL",
    expected = "Original and deliberately perturbed weights reach the fitting engine.",
    observed = paste0("original maximum difference = ", .format_num(.max_abs_diff(expected_original, observed_original)),
                      "; perturbed maximum difference = ", .format_num(.max_abs_diff(expected_perturbed, observed_perturbed))),
    details = if (ok) "The weight vectors reached the original and perturbed fits." else "The fitting engine did not preserve supplied weight semantics."
  )
}

.run_selected_test <- function(row, info, tolerance, seed, conf_level = 0.95, audit_level = "standard") {
  id <- row$id[[1L]]
  parameter <- row$parameter[[1L]]
  label <- row$test[[1L]]
  .safe_test(
    switch(
      id,
      row_order = .test_row_order(info, tolerance, seed),
      formula_order = .test_formula_order(info, tolerance),
      model_frame = .test_model_frame(info),
      prediction_reconstruction = .test_prediction_reconstruction(info, tolerance),
      weight_semantics = .test_weight_semantics(info, tolerance),
      predictor_rescaling = .test_predictor_rescaling(info, parameter, tolerance),
      factor_label_invariance = .test_factor_label_invariance(info, parameter, tolerance),
      focus_reference_reversal = .test_focus_reference(info, tolerance),
      outcome_translation = .test_outcome_translation(info, tolerance),
      outcome_scaling = .test_outcome_scaling(info, tolerance),
      known_effect_injection = .test_known_effect(info, tolerance),
      independent_qr = .test_independent_qr(info, tolerance),
      binary_outcome_reversal = .test_binary_outcome_reversal(info, tolerance),
      likelihood_reconstruction = .test_likelihood(info, tolerance),
      anova_cell_audit = .test_anova_cells(info),
      anova_term_order = .test_anova_term_order(info, tolerance),
      group_label_invariance = .test_group_label_invariance(info, parameter, tolerance),
      mixed_convergence = .test_mixed_convergence(info),
      mixed_singularity = .test_mixed_singularity(info, tolerance),
      optimizer_agreement = .test_optimizer_agreement(info, tolerance),
      random_structure = .test_random_structure(info),
      fixed_design_condition = .test_fixed_design_condition(info),
      design_support = .test_design_support(info),
      model_convergence = .test_model_convergence(info),
      residual_shape = .test_residual_shape(info),
      residual_variance = .test_residual_variance(info),
      glm_dispersion = .test_glm_dispersion(info),
      binomial_separation = .test_binomial_separation(info),
      case_influence = .test_case_influence(info),
      mixed_design_adequacy = .test_mixed_design_adequacy(info),
      group_influence = .test_group_influence(
        info, parameter, conf_level = conf_level,
        max_groups = if (identical(audit_level, "thorough")) 100L else 30L
      ),
      survival_time_scaling = .test_survival_time_scaling(info, tolerance),
      event_coding = .test_event_coding(info),
      cox_ph_assumption = .test_cox_ph_assumption(info),
      survival_information = .test_survival_information(info),
      survival_influence = .test_survival_influence(info),
      .sw_abort("Unknown witness test: ", id)
    ),
    id = id,
    label = label
  )
}

.certificate_status <- function(tests) {
  if (any(tests$status %in% c("FAIL", "ERROR") & tests$severity == "critical")) return("FAILED")
  if (any(tests$status %in% c("WARN", "FAIL", "ERROR"))) return("WARNING")
  "PASSED"
}

#' Generate a behavioral validation certificate
#'
#' Fits a supported statistical model, selects mathematically applicable
#' behavioral tests, applies controlled transformations, and checks whether
#' the analysis responds as expected.
#'
#' @param formula A model formula. Random-effect terms using `|` are supported
#'   when `lme4` is installed. `survival::Surv()` responses are supported when
#'   `survival` is installed.
#' @param data A data frame containing all analysis variables.
#' @param focus Optional coefficient, predictor, or omnibus term such as
#'   `"criterion:model"`. If omitted, the first estimable non-intercept
#'   coefficient is audited.
#' @param family Optional model family for GLM/GLMM analyses.
#' @param weights Optional numeric vector or data-column name.
#' @param method Fitting method: `"auto"`, `"lm"`, `"aov"`, `"glm"`,
#'   `"lmer"`, `"glmer"`, `"glmmTMB"`, `"coxph"`, or `"survreg"`.
#' @param engine Optional custom fitting function. It may accept any subset of
#'   `formula`, `data`, `family`, `weights`, `method`, and `control`.
#' @param control Optional model-specific control object.
#' @param conf_level Confidence level for coefficient intervals.
#' @param tolerance Relative numerical tolerance.
#' @param seed Seed used for deterministic perturbations.
#' @param include Optional character vector of test identifiers to retain.
#' @param exclude Optional character vector of test identifiers to omit.
#' @param audit_level Audit breadth: `"core"` runs integrity and numerical checks,
#'   `"standard"` adds assumptions and feasible influence checks, and
#'   `"thorough"` raises the limit for deletion-based group audits.
#' @return An object of class `statwitness_audit`.
#' @export
statwitness <- function(formula, data, focus = NULL, family = NULL,
                        weights = NULL, method = "auto", engine = NULL,
                        control = NULL, conf_level = 0.95,
                        tolerance = 1e-7, seed = 20260712,
                        include = NULL, exclude = NULL,
                        audit_level = c("standard", "core", "thorough")) {
  audit_level <- match.arg(audit_level)
  if (!inherits(formula, "formula")) .sw_abort("`formula` must be an R formula.")
  if (!is.data.frame(data)) .sw_abort("`data` must be a data frame.")
  if (!is.numeric(conf_level) || length(conf_level) != 1L || conf_level <= 0 || conf_level >= 1) {
    .sw_abort("`conf_level` must be a single number strictly between 0 and 1.")
  }
  if (!is.numeric(tolerance) || length(tolerance) != 1L || tolerance <= 0) {
    .sw_abort("`tolerance` must be a positive number.")
  }

  focus_expr <- substitute(focus)
  focus_value <- if (missing(focus) || identical(focus_expr, quote(NULL))) {
    NULL
  } else if (is.symbol(focus_expr) && as.character(focus_expr) %in% names(data)) {
    as.character(focus_expr)
  } else focus

  weights_expr <- substitute(weights)
  weights_vec <- if (missing(weights)) {
    NULL
  } else if (is.symbol(weights_expr) && as.character(weights_expr) %in% names(data)) {
    data[[as.character(weights_expr)]]
  } else .resolve_weights(data, weights, weights_expr, FALSE)

  resolved_method <- .detect_method(formula, family, method)
  model <- .fit_analysis(formula, data, family, weights_vec, resolved_method, engine, control)
  info <- .inspect_analysis(model, formula, data, focus_value, weights_vec, engine, resolved_method, control)
  primary <- .primary_result(info, conf_level)
  plan <- .build_plan(info, audit_level = audit_level)
  selected <- plan$selected
  if (!is.null(include)) selected <- selected[selected$id %in% include, , drop = FALSE]
  if (!is.null(exclude)) selected <- selected[!selected$id %in% exclude, , drop = FALSE]
  if (!nrow(selected)) .sw_abort("No validation tests remain after applying `include` and `exclude`.")

  results <- lapply(seq_len(nrow(selected)), function(i) {
    .run_selected_test(selected[i, , drop = FALSE], info, tolerance, seed + i, conf_level, audit_level)
  })
  tests <- do.call(rbind, lapply(results, .test_to_row))
  rownames(tests) <- NULL
  domains <- .domain_summary(tests)
  status <- .certificate_status(tests)
  data_fingerprint <- .md5_object(data)
  analysis_fingerprint <- .md5_object(list(
    formula = paste(deparse(formula), collapse = " "), method = resolved_method,
    focus = primary$label, family = info$family, link = info$link,
    coefficients = .fixed_coef(model), tests = tests,
    domains = domains, audit_level = audit_level,
    data_fingerprint = data_fingerprint
  ))
  certificate_id <- paste0("SW-", toupper(substr(analysis_fingerprint, 1L, 12L)))

  structure(list(
    call = match.call(), formula = formula, focus = info$focus,
    focus_coefficient = info$focus_coef, focus_term = info$focus_term,
    focus_kind = info$focus_kind, focus_inferred = info$focus_inferred,
    model = model, model_class = info$model_class, model_type = info$model_type,
    method = resolved_method, family = info$family, link = info$link,
    primary = primary, tests = tests, domains = domains,
    selected_plan = selected, skipped_plan = plan$skipped,
    status = status, audit_level = audit_level,
    certificate_id = certificate_id, data_fingerprint = data_fingerprint,
    analysis_fingerprint = analysis_fingerprint, tolerance = tolerance, seed = seed,
    limitations = c(
      "The certificate evaluates computational behavior and selected diagnostics under registered transformations.",
      "It does not establish causal validity, absence of confounding, data accuracy, or scientific appropriateness."
    )
  ), class = "statwitness_audit")
}

test_that("lmer interaction is audited as an omnibus term", {
  skip_if_not_installed("lme4")
  dat <- lme4::sleepstudy
  dat$criterion <- factor(rep(c("A", "B"), length.out = nrow(dat)))
  dat$model <- factor(rep(c("M1", "M2"), each = 2, length.out = nrow(dat)))
  dat$rater <- dat$Subject

  out <- statwitness(
    Reaction ~ criterion * model + (1 | rater),
    data = dat,
    focus = "criterion:model",
    exclude = "optimizer_agreement",
    audit_level = "core"
  )

  expect_s3_class(out, "statwitness_audit")
  expect_identical(out$model_class, "lmerMod")
  expect_identical(out$focus_kind, "term")
  expect_true(any(out$tests$id == "group_label_invariance"))
  expect_true(any(out$tests$id == "mixed_convergence"))
})

test_that("glmer is selected from a random-effects formula and family", {
  skip_if_not_installed("lme4")
  set.seed(1)
  dat <- data.frame(
    rater = factor(rep(seq_len(30), each = 8)),
    criterion = factor(rep(c("A", "B"), 120)),
    model = factor(rep(c("M1", "M2"), each = 2, length.out = 240))
  )
  b <- rnorm(30, 0, 0.5)[dat$rater]
  dat$correct <- rbinom(nrow(dat), 1, plogis(-0.2 + 0.4 * (dat$model == "M2") + b))

  out <- statwitness(
    correct ~ criterion * model + (1 | rater),
    data = dat,
    family = binomial(),
    focus = "criterion:model",
    exclude = "optimizer_agreement",
    audit_level = "core"
  )

  expect_identical(out$model_class, "glmerMod")
  expect_identical(out$method, "glmer")
})

test_that("binomial GLM receives model-specific tests", {
  set.seed(101)
  n <- 300
  dat <- data.frame(
    treatment = factor(sample(c("control", "new"), n, replace = TRUE)),
    age = rnorm(n, 60, 10),
    severity = rnorm(n)
  )
  eta <- -0.3 + 0.7 * (dat$treatment == "new") - 0.01 * dat$age - 0.4 * dat$severity
  dat$recovered <- rbinom(n, 1, plogis(eta))

  out <- statwitness(
    recovered ~ treatment + age + severity,
    data = dat,
    family = binomial(),
    focus = "treatment"
  )

  expect_s3_class(out, "statwitness_audit")
  expect_true(any(out$tests$id == "binary_outcome_reversal"))
  expect_true(any(out$tests$id == "likelihood_reconstruction"))
  expect_false(any(out$tests$status %in% c("FAIL", "ERROR")))
})

test_that("Poisson likelihood is reconstructed", {
  set.seed(202)
  n <- 220
  dat <- data.frame(x = rnorm(n), z = rnorm(n))
  dat$y <- rpois(n, exp(0.2 + 0.3 * dat$x - 0.1 * dat$z))

  out <- statwitness(y ~ x + z, data = dat, family = poisson(), focus = "x")
  ll <- out$tests[out$tests$id == "likelihood_reconstruction", , drop = FALSE]

  expect_equal(nrow(ll), 1L)
  expect_identical(ll$status, "PASS")
})

test_that("linear-model audit returns a certificate", {
  dat <- transform(mtcars, transmission = factor(am))

  out <- statwitness(
    mpg ~ transmission + wt + hp,
    data = dat,
    focus = "transmission"
  )

  expect_s3_class(out, "statwitness_audit")
  expect_true(out$status %in% c("PASSED", "WARNING"))
  expect_true(all(c("test", "status", "domain", "observed") %in% names(out$tests)))
  expect_true(any(out$tests$id == "known_effect_injection"))
  expect_match(out$certificate_id, "^SW-[A-F0-9]{12}$")
  expect_equal(nchar(out$data_fingerprint), 32L)
  expect_false(any(out$tests$status %in% c("FAIL", "ERROR")))
})

test_that("ANOVA-like factor design is detected", {
  out <- statwitness(
    breaks ~ wool * tension,
    data = warpbreaks,
    focus = "wool"
  )

  expect_identical(out$model_type, "anova_linear_model")
  expect_true(any(out$tests$id == "anova_cell_audit"))
  expect_false(any(out$tests$status %in% c("FAIL", "ERROR")))
})

test_that("bare weight column is supported", {
  dat <- transform(mtcars, w = seq_len(nrow(mtcars)) / nrow(mtcars))
  out <- statwitness(mpg ~ wt + hp, data = dat, focus = "wt", weights = w)
  expect_s3_class(out, "statwitness_audit")
  expect_false(any(out$tests$status %in% c("FAIL", "ERROR")))
})

test_that("custom engine that ignores weights is detected", {
  dat <- transform(mtcars, w = seq_len(nrow(mtcars)))
  bad_engine <- function(formula, data, family = NULL, weights = NULL) {
    stats::lm(formula, data = data, model = TRUE, x = TRUE, y = TRUE)
  }

  out <- statwitness(
    mpg ~ wt + hp,
    data = dat,
    focus = "wt",
    weights = w,
    engine = bad_engine
  )

  weight_test <- out$tests[out$tests$id == "weight_semantics", , drop = FALSE]
  expect_identical(weight_test$status, "FAIL")
  expect_identical(out$status, "FAILED")
})

test_that("Cox model receives survival-specific tests", {
  skip_if_not_installed("survival")
  dat <- survival::lung
  dat$status01 <- as.integer(dat$status == 2)
  out <- statwitness(
    survival::Surv(time, status01) ~ age + sex,
    data = dat,
    focus = "age",
    exclude = "factor_label_invariance"
  )
  expect_identical(out$model_class, "coxph")
  expect_true(any(out$tests$id == "survival_time_scaling"))
  expect_true(any(out$tests$id == "cox_ph_assumption"))
})

# Cross-model diagnostic and influence audits ---------------------------------

.test_fixed_design_condition <- function(info) {
  X <- tryCatch(.model_matrix_fixed(info$model), error = function(e) NULL)
  if (is.null(X) || !is.matrix(X) || !ncol(X)) {
    return(.new_test(
      "fixed_design_condition", "Fixed-effect design conditioning", "WARN",
      severity = "warning",
      expected = "A reconstructable fixed-effect design matrix.",
      observed = "The fixed-effect design matrix was unavailable.",
      details = "Conditioning and rank could not be evaluated automatically."
    ))
  }
  q <- qr(X)
  rank <- q$rank
  p <- ncol(X)
  cond <- tryCatch(kappa(X, exact = FALSE), error = function(e) Inf)
  aliased <- p - rank
  issue <- aliased > 0L || !is.finite(cond) || cond > 1e8
  .new_test(
    "fixed_design_condition", "Fixed-effect design conditioning",
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "Full column rank and no extreme numerical conditioning.",
    observed = paste0("columns = ", p, "; rank = ", rank,
                      "; aliased columns = ", aliased,
                      "; condition number = ", .format_num(cond)),
    details = if (issue) {
      "The fixed-effect design is rank deficient or extremely ill-conditioned; coefficients and omnibus tests may be unstable or non-identifiable."
    } else {
      "No fixed-effect rank deficiency or extreme conditioning was detected."
    }
  )
}

.finite_residuals <- function(model, type = NULL) {
  r <- tryCatch({
    if (is.null(type)) stats::residuals(model) else stats::residuals(model, type = type)
  }, error = function(e) numeric())
  r <- as.numeric(r)
  r[is.finite(r)]
}

.test_residual_shape <- function(info) {
  r <- .finite_residuals(info$model)
  n <- length(r)
  if (n < 8L || stats::sd(r) == 0) {
    return(.new_test(
      "residual_shape", "Residual-shape screening", "WARN", severity = "warning",
      expected = "Enough nonconstant residuals for a shape screen.",
      observed = paste0("usable residuals = ", n),
      details = "Residual shape could not be screened reliably."
    ))
  }
  z <- (r - mean(r)) / stats::sd(r)
  skew <- mean(z^3)
  excess <- mean(z^4) - 3
  issue <- abs(skew) > 2 || excess > 7
  .new_test(
    "residual_shape", "Residual-shape screening",
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "No extreme residual skewness or tail heaviness.",
    observed = paste0("skewness = ", .format_num(skew),
                      "; excess kurtosis = ", .format_num(excess),
                      "; residuals = ", n),
    details = if (issue) {
      "The residual distribution is strongly asymmetric or heavy-tailed. Inspect residual plots and consider a more appropriate outcome model or robust analysis."
    } else {
      "No extreme residual-shape problem was detected. This screen does not prove normality."
    }
  )
}

.test_residual_variance <- function(info) {
  r <- .finite_residuals(info$model)
  f <- tryCatch(as.numeric(stats::fitted(info$model)), error = function(e) numeric())
  n_common <- min(length(r), length(f))
  if (!n_common) {
    return(.new_test(
      "residual_variance", "Residual-variance screening", "WARN", severity = "warning",
      expected = "Matched residual and fitted-value vectors.",
      observed = "No matched residual/fitted values were available.",
      details = "Residual-variance heterogeneity could not be screened reliably."
    ))
  }
  r <- r[seq_len(n_common)]
  f <- f[seq_len(n_common)]
  keep <- is.finite(r) & is.finite(f)
  r <- r[keep]
  f <- f[keep]
  if (length(r) < 20L || length(unique(f)) < 4L) {
    return(.new_test(
      "residual_variance", "Residual-variance screening", "WARN", severity = "warning",
      expected = "Enough fitted-value variation to compare residual variance.",
      observed = paste0("usable observations = ", length(r),
                        "; unique fitted values = ", length(unique(f))),
      details = "Residual-variance heterogeneity could not be screened reliably."
    ))
  }
  breaks <- unique(stats::quantile(f, probs = seq(0, 1, length.out = 5), na.rm = TRUE))
  if (length(breaks) < 3L) {
    return(.new_test(
      "residual_variance", "Residual-variance screening", "WARN", severity = "warning",
      expected = "At least three fitted-value bins.",
      observed = paste0("distinct quantile breaks = ", length(breaks)),
      details = "Fitted values were too concentrated for the automatic variance screen."
    ))
  }
  bins <- cut(f, breaks = breaks, include.lowest = TRUE, labels = FALSE)
  vars <- vapply(split(r, bins), function(x) if (length(x) >= 3L) stats::var(x) else NA_real_, numeric(1))
  vars <- vars[is.finite(vars) & vars > 0]
  ratio <- if (length(vars) >= 2L) max(vars) / min(vars) else NA_real_
  issue <- is.finite(ratio) && ratio > 4
  .new_test(
    "residual_variance", "Residual-variance screening",
    if (issue) "WARN" else if (is.na(ratio)) "WARN" else "PASS",
    severity = "warning",
    expected = "No large residual-variance differences across fitted-value regions.",
    observed = paste0("largest/smallest binned residual variance ratio = ", .format_num(ratio)),
    details = if (issue) {
      "Residual variance differs substantially across fitted-value regions; inspect heteroskedasticity and group-specific variance."
    } else if (is.na(ratio)) {
      "Too few usable variance bins were available for evaluation."
    } else {
      "No large fitted-value-related variance difference was detected by the screening rule."
    }
  )
}

.test_model_convergence <- function(info) {
  if (info$model_class == "glm") {
    ok <- isTRUE(info$model$converged) && is.finite(info$model$deviance)
    return(.new_test(
      "model_convergence", "Model convergence audit", if (ok) "PASS" else "WARN",
      severity = "warning",
      expected = "The iterative fit reports convergence and finite deviance.",
      observed = paste0("converged = ", info$model$converged,
                        "; iterations = ", info$model$iter,
                        "; finite deviance = ", is.finite(info$model$deviance)),
      details = if (ok) "The GLM reported successful convergence." else "The GLM did not report a clean converged solution."
    ))
  }
  if (info$model_class %in% c("coxph", "survreg")) {
    beta <- .fixed_coef(info$model)
    finite <- all(is.finite(beta))
    iter <- info$model$iter %||% NA_integer_
    return(.new_test(
      "model_convergence", "Model convergence audit", if (finite) "PASS" else "WARN",
      severity = "warning",
      expected = "Finite fitted coefficients and no reported fitting failure.",
      observed = paste0("finite coefficients = ", finite,
                        "; iterations = ", paste(iter, collapse = ", ")),
      details = if (finite) "No non-finite fitted coefficient was detected." else "At least one coefficient is non-finite; the survival fit requires review."
    ))
  }
  .new_test(
    "model_convergence", "Model convergence audit", "PASS", severity = "warning",
    expected = "A closed-form or directly solved fit without iterative convergence failure.",
    observed = "No model-specific iterative convergence failure applies.",
    details = "This model is not audited through an iterative convergence flag."
  )
}

.test_glm_dispersion <- function(info) {
  r <- .finite_residuals(info$model, type = "pearson")
  df <- tryCatch(stats::df.residual(info$model), error = function(e) NA_real_)
  dispersion <- if (length(r) && is.finite(df) && df > 0) sum(r^2) / df else NA_real_
  high <- is.finite(dispersion) && dispersion > 1.5
  low <- info$family == "poisson" && is.finite(dispersion) && dispersion < 0.5
  issue <- high || low
  .new_test(
    "glm_dispersion", "Generalized-model dispersion audit",
    if (issue) "WARN" else if (!is.finite(dispersion)) "WARN" else "PASS",
    severity = "warning",
    expected = "Pearson dispersion reasonably compatible with the declared mean-variance family.",
    observed = paste0("Pearson dispersion = ", .format_num(dispersion),
                      "; residual df = ", .format_num(df)),
    details = if (high) {
      "Substantial overdispersion was detected; standard errors and the response family may require review."
    } else if (low) {
      "Substantial underdispersion was detected relative to a Poisson model."
    } else if (!is.finite(dispersion)) {
      "Dispersion could not be computed automatically."
    } else {
      "No large dispersion discrepancy was detected."
    }
  )
}

.test_binomial_separation <- function(info) {
  beta <- .fixed_coef(info$model)
  V <- tryCatch(.fixed_vcov(info$model), error = function(e) matrix(NA_real_, 0, 0))
  se <- if (length(V)) sqrt(diag(V)) else rep(NA_real_, length(beta))
  fitted <- tryCatch(.model_response_prediction(info$model), error = function(e) numeric())
  extreme <- if (length(fitted)) mean(fitted < 1e-8 | fitted > 1 - 1e-8, na.rm = TRUE) else NA_real_
  issue <- any(!is.finite(beta)) || any(abs(beta) > 20, na.rm = TRUE) ||
    any(se > 10, na.rm = TRUE) || (is.finite(extreme) && extreme > 0.25)
  max_beta <- if (any(is.finite(beta))) max(abs(beta[is.finite(beta)])) else Inf
  max_se <- if (any(is.finite(se))) max(se[is.finite(se)]) else Inf
  .new_test(
    "binomial_separation", "Binomial separation screening",
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "Finite, moderately scaled coefficients without widespread boundary probabilities.",
    observed = paste0("maximum |coefficient| = ", .format_num(max_beta),
                      "; maximum SE = ", .format_num(max_se),
                      "; boundary-probability fraction = ", .format_num(extreme)),
    details = if (issue) {
      "The fit shows signs compatible with complete/quasi separation or weakly identified binomial effects. Confirm with a dedicated separation diagnostic."
    } else {
      "No strong numerical sign of binomial separation was detected."
    }
  )
}

.test_case_influence <- function(info) {
  cd <- tryCatch(stats::cooks.distance(info$model), error = function(e) numeric())
  cd <- as.numeric(cd)
  cd <- cd[is.finite(cd)]
  n <- length(cd)
  if (!n) {
    return(.new_test(
      "case_influence", "Case-level influence screening", "WARN", severity = "warning",
      expected = "Available case-deletion influence diagnostics.",
      observed = "Cook's distance was unavailable.",
      details = "Case-level influence could not be screened automatically for this fitted object."
    ))
  }
  threshold <- 4 / n
  n_flag <- sum(cd > threshold)
  max_cd <- max(cd)
  issue <- max_cd > 1 || n_flag > max(2L, ceiling(0.05 * n))
  .new_test(
    "case_influence", "Case-level influence screening",
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "No single case with Cook's distance above 1 and no large concentration above 4/n.",
    observed = paste0("maximum Cook's distance = ", .format_num(max_cd),
                      "; cases above 4/n = ", n_flag, " of ", n),
    details = if (issue) {
      "Potentially influential observations were detected; inspect case-deletion estimates and data quality."
    } else {
      "No strong case-level influence signal was detected by Cook's-distance screening."
    }
  )
}

.random_slope_map <- function(formula) {
  if (!.has_random_terms(formula)) return(list())
  .require_namespace("reformulas", "mixed-model design inspection")
  bars <- reformulas::findbars(formula)
  out <- list()
  for (bar in bars) {
    groups <- all.vars(bar[[3L]])
    slopes <- all.vars(bar[[2L]])
    for (g in groups) out[[g]] <- unique(c(out[[g]], slopes))
  }
  out
}

.test_mixed_design_adequacy <- function(info) {
  slope_map <- .random_slope_map(info$formula)
  findings <- character()
  for (g in info$random_groups) {
    if (!g %in% names(info$data)) next
    slopes <- slope_map[[g]] %||% character()
    candidates <- setdiff(info$simple_predictors, c(g, slopes))
    for (v in candidates) {
      split_values <- split(info$data[[v]], info$data[[g]], drop = TRUE)
      varies <- vapply(split_values, function(x) length(unique(stats::na.omit(x))) > 1L, logical(1))
      proportion <- if (length(varies)) mean(varies) else 0
      if (proportion >= 0.5) {
        findings <- c(findings, paste0(v, " varies within ", g,
                                      " for ", round(100 * proportion),
                                      "% of groups but has no registered random slope"))
      }
    }
  }
  issue <- length(findings) > 0L
  .new_test(
    "mixed_design_adequacy", "Random-effects design audit",
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "Within-cluster predictors are identified and compared with the specified random-slope structure.",
    observed = if (issue) paste(findings, collapse = "; ") else "No obvious omitted within-group slope candidate was detected.",
    details = if (issue) {
      "This is a design review item, not an automatic instruction to add every possible random slope. Compare scientifically defensible random-effects structures."
    } else {
      "The automatic design screen found no simple within-group predictor omitted from all corresponding random-slope terms."
    }
  )
}

.focus_snapshot <- function(info, conf_level = 0.95) {
  out <- tryCatch(.primary_result(info, conf_level), error = function(e) NULL)
  if (is.null(out)) return(NULL)
  if (out$kind == "term") {
    list(kind = "term", statistic = out$statistic, p_value = out$p_value)
  } else {
    list(kind = "coefficient", estimate = out$estimate, p_value = out$p_value)
  }
}

.test_group_influence <- function(info, variable, conf_level = 0.95, max_groups = 30L) {
  groups <- unique(stats::na.omit(info$data[[variable]]))
  if (length(groups) < 3L) {
    return(.new_test(
      "group_influence", paste0("Group-deletion influence: ", variable), "WARN",
      severity = "warning",
      expected = "At least three groups for deletion-based influence screening.",
      observed = paste0("groups = ", length(groups)),
      details = "Group-deletion influence was not evaluated."
    ))
  }
  if (length(groups) > max_groups) {
    return(.new_test(
      "group_influence", paste0("Group-deletion influence: ", variable), "WARN",
      severity = "warning",
      expected = paste0("At most ", max_groups, " groups in the selected audit level."),
      observed = paste0("groups = ", length(groups)),
      details = "The complete leave-one-group-out audit was skipped to control computation; use audit_level = 'thorough' to raise the limit."
    ))
  }
  base <- .focus_snapshot(info, conf_level)
  if (is.null(base)) {
    return(.new_test("group_influence", paste0("Group-deletion influence: ", variable), "WARN",
      severity = "warning", expected = "A reproducible primary focus result.",
      observed = "The primary result could not be reconstructed for influence analysis.",
      details = "Group-deletion influence was not evaluated."))
  }
  changes <- numeric()
  pvals <- numeric()
  reversals <- logical()
  failures <- 0L
  labels <- character()
  for (g in groups) {
    keep <- !is.na(info$data[[variable]]) & info$data[[variable]] != g
    d2 <- info$data[keep, , drop = FALSE]
    w2 <- if (is.null(info$weights)) NULL else info$weights[keep]
    snap <- tryCatch({
      m2 <- .refit(info, d2, weights = w2)
      focus_arg <- if (info$focus_kind == "term") info$focus_term else info$focus_coef
      info2 <- .inspect_analysis(m2, info$formula, d2, focus_arg, w2,
                                info$engine, info$method, info$control)
      .focus_snapshot(info2, conf_level)
    }, error = function(e) NULL)
    if (is.null(snap)) {
      failures <- failures + 1L
      next
    }
    labels <- c(labels, as.character(g))
    pvals <- c(pvals, snap$p_value)
    if (base$kind == "coefficient") {
      denom <- max(abs(base$estimate), 1e-8)
      changes <- c(changes, abs(snap$estimate - base$estimate) / denom)
      reversals <- c(reversals, sign(snap$estimate) != sign(base$estimate))
    } else {
      denom <- max(abs(base$statistic), 1e-8)
      changes <- c(changes, abs(snap$statistic - base$statistic) / denom)
      reversals <- c(reversals, (snap$p_value < 0.05) != (base$p_value < 0.05))
    }
  }
  if (!length(changes)) {
    return(.new_test("group_influence", paste0("Group-deletion influence: ", variable), "WARN",
      severity = "warning", expected = "At least one successful leave-one-group-out refit.",
      observed = paste0("failed refits = ", failures),
      details = "No group-deletion refit completed successfully."))
  }
  idx <- which.max(changes)
  issue <- any(reversals) || max(changes) > 0.5
  .new_test(
    "group_influence", paste0("Group-deletion influence: ", variable),
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "The focus result remains directionally/decisionally stable and changes by no more than 50% after deleting one group.",
    observed = paste0("largest relative focus change = ", .format_num(max(changes)),
                      " after deleting ", labels[[idx]],
                      "; conclusion reversals = ", sum(reversals),
                      "; failed refits = ", failures),
    details = if (issue) {
      "The focus result is sensitive to at least one grouping level; inspect leave-one-group-out estimates and that group's data."
    } else {
      "No single grouping level caused a large focus change or decision reversal."
    }
  )
}

.test_survival_information <- function(info) {
  parts <- .surv_response_parts(info$formula)
  if (length(parts) < 2L || !parts[[2L]] %in% names(info$data)) {
    return(.new_test(
      "survival_information", "Survival information audit", "WARN", severity = "warning",
      expected = "An identifiable event indicator.",
      observed = "Event counts could not be reconstructed.",
      details = "The amount of event information per fitted coefficient was not evaluated."
    ))
  }
  event <- info$data[[parts[[2L]]]]
  events <- sum(event == 1, na.rm = TRUE)
  p <- sum(is.finite(.fixed_coef(info$model)))
  ratio <- if (p > 0) events / p else NA_real_
  issue <- is.finite(ratio) && ratio < 5
  .new_test(
    "survival_information", "Survival information audit",
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "Enough observed events to support the fitted coefficient count.",
    observed = paste0("events = ", events, "; fitted coefficients = ", p,
                      "; events per coefficient = ", .format_num(ratio)),
    details = if (issue) {
      "Very limited event information was detected relative to model size; estimates may be unstable. This is a screening threshold, not a universal sample-size rule."
    } else {
      "No very low event-to-parameter ratio was detected by the screening rule."
    }
  )
}

.test_survival_influence <- function(info) {
  d <- tryCatch(stats::residuals(info$model, type = "dfbeta"), error = function(e) NULL)
  if (is.null(d)) {
    return(.new_test(
      "survival_influence", "Survival influence screening", "WARN", severity = "warning",
      expected = "Available DFBETA residuals.", observed = "DFBETA residuals were unavailable.",
      details = "Observation-level survival influence could not be screened automatically."
    ))
  }
  d <- as.matrix(d)
  n <- nrow(d)
  threshold <- if (n > 0) 2 / sqrt(n) else NA_real_
  max_d <- max(abs(d), na.rm = TRUE)
  n_flag <- sum(apply(abs(d), 1L, max, na.rm = TRUE) > threshold)
  issue <- is.finite(max_d) && max_d > max(0.5, threshold * 3)
  .new_test(
    "survival_influence", "Survival influence screening",
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "No observation with an unusually large DFBETA profile.",
    observed = paste0("maximum |DFBETA| = ", .format_num(max_d),
                      "; rows above 2/sqrt(n) = ", n_flag, " of ", n),
    details = if (issue) "At least one observation may strongly influence a survival coefficient." else "No extreme DFBETA influence signal was detected."
  )
}

.domain_status <- function(test_rows) {
  if (!nrow(test_rows)) return("NOT EVALUATED")
  if (any(test_rows$status %in% c("FAIL", "ERROR") & test_rows$severity == "critical")) return("FAILED")
  if (any(test_rows$status %in% c("WARN", "FAIL", "ERROR"))) return("REVIEW")
  "PASSED"
}

.domain_summary <- function(tests) {
  domains <- c(
    "computational_integrity", "numerical_stability", "design_adequacy",
    "assumption_screening", "influence_stability"
  )
  extra <- setdiff(unique(tests$domain), domains)
  domains <- c(domains, extra)
  out <- do.call(rbind, lapply(domains, function(d) {
    x <- tests[tests$domain == d, , drop = FALSE]
    data.frame(
      domain = d,
      status = .domain_status(x),
      passed = sum(x$status == "PASS"),
      review = sum(x$status %in% c("WARN", "FAIL", "ERROR")),
      stringsAsFactors = FALSE
    )
  }))
  rownames(out) <- NULL
  out
}

.test_design_support <- function(info) {
  n <- tryCatch(.model_nobs(info$model), error = function(e) NA_integer_)
  beta <- tryCatch(.fixed_coef(info$model), error = function(e) numeric())
  p <- sum(is.finite(beta))
  rdf <- tryCatch(stats::df.residual(info$model), error = function(e) NA_real_)
  min_level <- Inf
  level_label <- NA_character_
  if (length(info$factor_predictors)) {
    for (v in info$factor_predictors) {
      tab <- table(info$data[[v]], useNA = "no")
      if (length(tab) && min(tab) < min_level) {
        min_level <- min(tab)
        level_label <- v
      }
    }
  }
  if (!is.finite(min_level)) min_level <- NA_real_
  severe <- (is.finite(n) && p > 0L && n <= p) ||
    (is.finite(rdf) && rdf <= 0) ||
    (is.finite(min_level) && min_level < 2)
  .new_test(
    "design_support", "Basic design-support audit",
    if (severe) "WARN" else "PASS", severity = "warning",
    expected = "More analysed observations than estimable fixed coefficients and no singleton fixed-factor level.",
    observed = paste0("observations = ", n,
                      "; estimable fixed coefficients = ", p,
                      "; residual df = ", .format_num(rdf),
                      "; smallest fixed-factor level = ", .format_num(min_level),
                      if (!is.na(level_label)) paste0(" (", level_label, ")") else ""),
    details = if (severe) {
      "The fitted design has extremely limited information, nonpositive residual degrees of freedom, or a singleton factor level."
    } else {
      "No severe basic design-support problem was detected. This is not a formal power or sample-size assessment."
    }
  )
}

.rm_fit <- function(data, outcome, id, within, between, type, correction) {
  .require_namespace("afex", "repeated-measures ANOVA")
  afex::aov_ez(
    id = id, dv = outcome, data = data,
    within = within, between = between,
    type = type, anova_table = list(correction = correction, es = "pes"),
    factorize = TRUE, return = "afex_aov"
  )
}

.rm_table <- function(model) {
  as.data.frame(model$anova_table)
}

.rm_numeric_column <- function(tab, patterns) {
  nm <- colnames(tab)
  hit <- unique(unlist(lapply(patterns, function(p) grep(p, nm, ignore.case = TRUE, value = TRUE))))
  if (!length(hit)) return(rep(NA_real_, nrow(tab)))
  suppressWarnings(as.numeric(tab[[hit[[1L]]]]))
}

.rm_effect_names <- function(tab) {
  rn <- rownames(tab)
  if (!is.null(rn) && all(nzchar(rn))) return(rn)
  if ("Effect" %in% names(tab)) return(as.character(tab$Effect))
  paste0("Effect_", seq_len(nrow(tab)))
}

.rm_primary <- function(model, focus) {
  tab <- .rm_table(model)
  effects <- .rm_effect_names(tab)
  if (is.null(focus)) focus <- effects[[1L]]
  idx <- match(focus, effects)
  if (is.na(idx)) .sw_abort("Could not match repeated-measures focus `", focus, "`. Available effects: ", paste(effects, collapse = ", "), ".")
  fval <- .rm_numeric_column(tab, c("^F$", "F.value", "^F "))
  pval <- .rm_numeric_column(tab, c("Pr", "p.value", "^p$"))
  ndf <- .rm_numeric_column(tab, c("num.Df", "num df", "^df1$"))
  ddf <- .rm_numeric_column(tab, c("den.Df", "den df", "^df2$"))
  pes <- .rm_numeric_column(tab, c("pes", "partial"))
  list(
    kind = "term", label = focus, coefficient = NA_character_, estimate = pes[[idx]],
    standard_error = NA_real_, statistic = fval[[idx]], df = c(ndf[[idx]], ddf[[idx]]),
    p_value = pval[[idx]], conf_level = NA_real_, lower = NA_real_, upper = NA_real_,
    transformed = NULL, n = NA_integer_,
    method = "afex repeated-measures ANOVA", effect_size_label = "partial eta-squared"
  )
}

.rm_compare_tables <- function(m1, m2, tolerance) {
  t1 <- .rm_table(m1)
  t2 <- .rm_table(m2)
  e1 <- .rm_effect_names(t1)
  e2 <- .rm_effect_names(t2)
  if (!identical(e1, e2)) return(list(ok = FALSE, diff = Inf))
  f1 <- .rm_numeric_column(t1, c("^F$", "F.value", "^F "))
  f2 <- .rm_numeric_column(t2, c("^F$", "F.value", "^F "))
  p1 <- .rm_numeric_column(t1, c("Pr", "p.value", "^p$"))
  p2 <- .rm_numeric_column(t2, c("Pr", "p.value", "^p$"))
  diffs <- c(.max_abs_diff(f1, f2), .max_abs_diff(p1, p2))
  diff <- if (all(is.na(diffs))) Inf else max(diffs, na.rm = TRUE)
  list(ok = .within_tol(f1, f2, tolerance * 100) && .within_tol(p1, p2, tolerance * 100), diff = diff)
}

.rm_test_design <- function(data, id, within) {
  key_vars <- c(id, within)
  key <- interaction(data[key_vars], drop = TRUE, lex.order = TRUE)
  duplicate_cells <- sum(duplicated(key))
  expected <- do.call(expand.grid, lapply(data[within], function(x) unique(stats::na.omit(x))))
  expected_n <- nrow(expected)
  counts <- table(data[[id]])
  incomplete_ids <- sum(counts != expected_n)
  status <- if (duplicate_cells > 0 || incomplete_ids > 0) "WARN" else "PASS"
  .new_test(
    "rm_design_integrity", "Repeated-measures design integrity", status, severity = "warning",
    expected = "One observation per participant-condition cell and complete within-subject cells.",
    observed = paste0("duplicate participant-condition rows = ", duplicate_cells,
                      "; participants with unexpected row counts = ", incomplete_ids),
    details = if (status == "PASS") "The long-format repeated design has one complete row per participant-condition cell." else "Duplicate or incomplete participant-condition cells were detected; verify aggregation and missingness handling."
  )
}

#' Audit a repeated-measures ANOVA
#'
#' Uses `afex::aov_ez()` and applies design-integrity and metamorphic checks.
#'
#' @param data Long-format data frame.
#' @param outcome Name of the numeric outcome column.
#' @param id Name of the participant identifier column.
#' @param within Character vector of within-subject factor names.
#' @param between Optional character vector of between-subject factor names.
#' @param focus Optional omnibus effect name, for example `"criterion:model"`.
#' @param type Sums-of-squares type passed to `afex::aov_ez()`.
#' @param correction Sphericity correction, usually `"GG"`.
#' @param tolerance Relative numerical tolerance.
#' @param seed Seed for row-order checks.
#' @param audit_level Audit breadth: `"core"`, `"standard"`, or `"thorough"`.
#' @return An object of class `statwitness_audit`.
#' @export
statwitness_repeated <- function(data, outcome, id, within, between = NULL,
                                 focus = NULL, type = 3, correction = "GG",
                                 tolerance = 1e-7, seed = 20260712,
                                 audit_level = c("standard", "core", "thorough")) {
  audit_level <- match.arg(audit_level)
  if (!is.data.frame(data)) .sw_abort("`data` must be a data frame.")
  args <- c(outcome, id, within, between)
  missing_vars <- setdiff(args, names(data))
  if (length(missing_vars)) .sw_abort("Missing columns: ", paste(missing_vars, collapse = ", "), ".")
  if (!is.numeric(data[[outcome]])) .sw_abort("`outcome` must identify a numeric column.")
  model <- .rm_fit(data, outcome, id, within, between, type, correction)
  primary <- .rm_primary(model, focus)
  primary$n <- length(unique(data[[id]]))

  tests <- list(.rm_test_design(data, id, within))
  tests[[length(tests) + 1L]] <- .rm_test_sphericity(data, within, correction)
  if (audit_level != "core") {
    tests[[length(tests) + 1L]] <- .rm_test_residual_shape(model)
    tests[[length(tests) + 1L]] <- .rm_test_id_influence(
      data, outcome, id, within, between, type, correction, primary$label,
      model, max_ids = if (identical(audit_level, "thorough")) 100L else 25L
    )
  }
  set.seed(seed)
  shuffled <- data[sample.int(nrow(data)), , drop = FALSE]
  m_row <- .rm_fit(shuffled, outcome, id, within, between, type, correction)
  cmp <- .rm_compare_tables(model, m_row, tolerance)
  tests[[length(tests) + 1L]] <- .new_test(
    "rm_row_order", "Repeated-ANOVA row-order invariance", if (cmp$ok) "PASS" else "FAIL",
    expected = "Equivalent F tests and p-values after row permutation.",
    observed = paste0("maximum ANOVA-table difference = ", .format_num(cmp$diff)),
    details = if (cmp$ok) "The repeated-measures ANOVA was invariant to row order." else "Row ordering changed the repeated-measures ANOVA."
  )

  relabeled <- data
  relabeled[[id]] <- factor(relabeled[[id]])
  levels(relabeled[[id]]) <- paste0("SW_ID_", seq_along(levels(relabeled[[id]])))
  m_id <- .rm_fit(relabeled, outcome, id, within, between, type, correction)
  cmp <- .rm_compare_tables(model, m_id, tolerance)
  tests[[length(tests) + 1L]] <- .new_test(
    "rm_id_label", "Participant-label invariance", if (cmp$ok) "PASS" else "FAIL",
    expected = "Renaming participant identifiers preserves omnibus tests.",
    observed = paste0("maximum ANOVA-table difference = ", .format_num(cmp$diff)),
    details = if (cmp$ok) "Participant labels were treated as identifiers." else "Renaming participant identifiers changed the ANOVA."
  )

  translated <- data
  shift <- stats::sd(translated[[outcome]], na.rm = TRUE)
  if (!is.finite(shift) || shift == 0) shift <- 1
  translated[[outcome]] <- translated[[outcome]] + shift
  m_trans <- .rm_fit(translated, outcome, id, within, between, type, correction)
  cmp <- .rm_compare_tables(model, m_trans, tolerance)
  tests[[length(tests) + 1L]] <- .new_test(
    "rm_outcome_translation", "Repeated-outcome translation", if (cmp$ok) "PASS" else "FAIL",
    expected = "Adding a constant preserves F tests and p-values.",
    observed = paste0("maximum ANOVA-table difference = ", .format_num(cmp$diff)),
    details = if (cmp$ok) "Outcome translation preserved the repeated-measures tests." else "Outcome translation altered the repeated-measures tests."
  )

  scaled <- data
  scaled[[outcome]] <- scaled[[outcome]] * 10
  m_scale <- .rm_fit(scaled, outcome, id, within, between, type, correction)
  cmp <- .rm_compare_tables(model, m_scale, tolerance)
  tests[[length(tests) + 1L]] <- .new_test(
    "rm_outcome_scaling", "Repeated-outcome scaling", if (cmp$ok) "PASS" else "FAIL",
    expected = "Changing outcome units preserves F tests and p-values.",
    observed = paste0("maximum ANOVA-table difference = ", .format_num(cmp$diff)),
    details = if (cmp$ok) "Outcome scaling preserved the repeated-measures tests." else "Outcome scaling altered the repeated-measures tests."
  )

  test_df <- do.call(rbind, lapply(tests, .test_to_row))
  domains <- .domain_summary(test_df)
  status <- .certificate_status(test_df)
  data_fingerprint <- .md5_object(data)
  analysis_fingerprint <- .md5_object(list(
    outcome = outcome, id = id, within = within, between = between,
    focus = primary$label, type = type, correction = correction,
    tests = test_df, domains = domains, audit_level = audit_level,
    data_fingerprint = data_fingerprint
  ))

  structure(list(
    call = match.call(), formula = NULL, focus = primary$label,
    focus_coefficient = NA_character_, focus_term = primary$label,
    focus_kind = "term", focus_inferred = is.null(focus), model = model,
    model_class = "afex_aov", model_type = "repeated_measures_anova",
    method = "afex_aov_ez", family = "gaussian", link = "identity",
    primary = primary, tests = test_df, domains = domains,
    selected_plan = NULL, skipped_plan = NULL, status = status,
    audit_level = audit_level,
    certificate_id = paste0("SW-", toupper(substr(analysis_fingerprint, 1L, 12L))),
    data_fingerprint = data_fingerprint,
    analysis_fingerprint = analysis_fingerprint,
    tolerance = tolerance, seed = seed,
    repeated_design = list(outcome = outcome, id = id, within = within,
                           between = between, type = type, correction = correction),
    limitations = c(
      "The certificate validates repeated-measures computation and design structure under registered transformations.",
      "It does not establish causal validity, sphericity truth, or scientific appropriateness."
    )
  ), class = "statwitness_audit")
}

.rm_test_sphericity <- function(data, within, correction) {
  levels_n <- vapply(within, function(v) length(unique(stats::na.omit(data[[v]]))), integer(1))
  relevant <- any(levels_n > 2L)
  declared <- !is.null(correction) && nzchar(as.character(correction)) &&
    !tolower(as.character(correction)) %in% c("none", "no", "false")
  issue <- relevant && !declared
  .new_test(
    "rm_sphericity", "Repeated-measures sphericity handling",
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "A declared correction when a within-subject factor has more than two levels.",
    observed = paste0("within-factor levels = ",
                      paste(names(levels_n), levels_n, sep = ":", collapse = ", "),
                      "; correction = ", correction %||% "none"),
    details = if (issue) {
      "At least one within-subject factor has more than two levels but no sphericity correction was declared."
    } else if (relevant) {
      "A sphericity correction was declared for the repeated-measures analysis."
    } else {
      "No within-subject factor has more than two levels, so sphericity correction is not required for a single-factor contrast."
    }
  )
}

.rm_test_residual_shape <- function(model) {
  lm_part <- tryCatch(model$lm, error = function(e) NULL)
  if (is.null(lm_part)) {
    return(.new_test(
      "rm_residual_shape", "Repeated-model residual-shape screening", "WARN",
      severity = "warning", expected = "An accessible fitted linear-model component.",
      observed = "The residual model component was unavailable.",
      details = "Residual shape could not be screened automatically."
    ))
  }
  r <- tryCatch(as.numeric(stats::residuals(lm_part)), error = function(e) numeric())
  r <- r[is.finite(r)]
  if (length(r) < 8L || stats::sd(r) == 0) {
    return(.new_test(
      "rm_residual_shape", "Repeated-model residual-shape screening", "WARN",
      severity = "warning", expected = "Enough nonconstant residuals.",
      observed = paste0("usable residuals = ", length(r)),
      details = "Residual shape could not be screened reliably."
    ))
  }
  z <- (r - mean(r)) / stats::sd(r)
  skew <- mean(z^3)
  excess <- mean(z^4) - 3
  issue <- abs(skew) > 2 || excess > 7
  .new_test(
    "rm_residual_shape", "Repeated-model residual-shape screening",
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "No extreme residual skewness or tail heaviness.",
    observed = paste0("skewness = ", .format_num(skew),
                      "; excess kurtosis = ", .format_num(excess)),
    details = if (issue) "The repeated-model residuals are strongly asymmetric or heavy-tailed." else "No extreme residual-shape issue was detected."
  )
}

.rm_test_id_influence <- function(data, outcome, id, within, between, type,
                                  correction, focus, base_model,
                                  max_ids = 25L) {
  ids <- unique(stats::na.omit(data[[id]]))
  if (length(ids) > max_ids) {
    return(.new_test(
      "rm_id_influence", "Participant-deletion influence", "WARN",
      severity = "warning", expected = paste0("At most ", max_ids, " participants in the selected audit level."),
      observed = paste0("participants = ", length(ids)),
      details = "The complete leave-one-participant-out audit was skipped to control computation; use audit_level = 'thorough' to raise the limit."
    ))
  }
  base <- .rm_primary(base_model, focus)
  changes <- numeric()
  reversals <- logical()
  labels <- character()
  failures <- 0L
  for (who in ids) {
    d2 <- data[data[[id]] != who, , drop = FALSE]
    snap <- tryCatch(.rm_primary(.rm_fit(d2, outcome, id, within, between, type, correction), focus),
                     error = function(e) NULL)
    if (is.null(snap)) {
      failures <- failures + 1L
      next
    }
    denom <- max(abs(base$statistic), 1e-8)
    changes <- c(changes, abs(snap$statistic - base$statistic) / denom)
    reversals <- c(reversals, (snap$p_value < 0.05) != (base$p_value < 0.05))
    labels <- c(labels, as.character(who))
  }
  if (!length(changes)) {
    return(.new_test(
      "rm_id_influence", "Participant-deletion influence", "WARN",
      severity = "warning", expected = "At least one successful leave-one-participant-out refit.",
      observed = paste0("failed refits = ", failures),
      details = "No participant-deletion refit completed successfully."
    ))
  }
  idx <- which.max(changes)
  issue <- any(reversals) || max(changes) > 0.5
  .new_test(
    "rm_id_influence", "Participant-deletion influence",
    if (issue) "WARN" else "PASS", severity = "warning",
    expected = "The focus test remains decisionally stable and changes by no more than 50% after deleting one participant.",
    observed = paste0("largest relative statistic change = ", .format_num(max(changes)),
                      " after deleting ", labels[[idx]],
                      "; decision reversals = ", sum(reversals),
                      "; failed refits = ", failures),
    details = if (issue) "The repeated-measures result is sensitive to at least one participant." else "No single participant caused a large test-statistic change or decision reversal."
  )
}

test_that("repeated-measures ANOVA audit returns a certificate", {
  skip_if_not_installed("afex")
  set.seed(3)
  dat <- expand.grid(
    rater = factor(seq_len(24)),
    criterion = factor(c("accuracy", "clarity")),
    model = factor(c("A", "B"))
  )
  dat$score <- rnorm(nrow(dat), 70, 8) + 3 * (dat$model == "B")

  out <- statwitness_repeated(
    data = dat,
    outcome = "score",
    id = "rater",
    within = c("criterion", "model"),
    focus = "criterion:model",
    audit_level = "core"
  )

  expect_s3_class(out, "statwitness_audit")
  expect_identical(out$model_type, "repeated_measures_anova")
  expect_true(any(out$tests$id == "rm_design_integrity"))
})

.surv_response_parts <- function(formula) {
  lhs <- formula[[2L]]
  if (!is.call(lhs)) return(character())
  fn <- paste(deparse(lhs[[1L]]), collapse = "")
  if (!fn %in% c("Surv", "survival::Surv")) return(character())
  vapply(as.list(lhs)[-1L], function(x) paste(deparse(x), collapse = ""), character(1))
}

.test_survival_time_scaling <- function(info, tolerance) {
  parts <- .surv_response_parts(info$formula)
  if (length(parts) < 2L || !parts[[1L]] %in% names(info$data)) {
    return(.new_test("survival_time_scaling", "Survival-time unit scaling", "ERROR",
      expected = "A direct Surv(time, event) response.", observed = "The survival time variable could not be identified.",
      details = "Time scaling currently supports direct column names in Surv()."))
  }
  time_var <- parts[[1L]]
  d2 <- info$data
  d2[[time_var]] <- d2[[time_var]] * 10
  m2 <- .refit(info, d2)
  ov <- .named_overlap(.fixed_coef(info$model), .fixed_coef(m2))
  ok <- .within_tol(ov$x, ov$y, tolerance * 100)
  .new_test(
    "survival_time_scaling", "Survival-time unit scaling", if (ok) "PASS" else "FAIL",
    expected = "Changing the time unit preserves regression coefficients and hazard ratios.",
    observed = paste0("maximum coefficient difference = ", .format_num(.max_abs_diff(ov$x, ov$y))),
    details = if (ok) "The survival model was invariant to time-unit conversion." else "Changing time units altered the regression effects."
  )
}

.test_event_coding <- function(info) {
  parts <- .surv_response_parts(info$formula)
  if (length(parts) < 2L || !parts[[2L]] %in% names(info$data)) {
    return(.new_test("event_coding", "Event-coding audit", "WARN", severity = "warning",
      expected = "A direct event indicator is identifiable.", observed = "Event coding could not be inspected automatically.",
      details = "Inspect the Surv() response and event coding manually."))
  }
  event <- info$data[[parts[[2L]]]]
  vals <- sort(unique(stats::na.omit(event)))
  ok <- all(vals %in% c(0, 1)) && length(vals) == 2L
  .new_test(
    "event_coding", "Event-coding audit", if (ok) "PASS" else "WARN", severity = "warning",
    expected = "A two-valued 0/1 event indicator for right-censored data.",
    observed = paste0("observed event values: ", paste(vals, collapse = ", ")),
    details = if (ok) "The event indicator uses conventional 0/1 coding." else "Nonstandard or single-valued event coding was detected; verify the Surv() semantics."
  )
}

.test_cox_ph_assumption <- function(info) {
  if (info$model_class != "coxph") {
    return(.new_test("cox_ph_assumption", "Proportional-hazards diagnostic", "WARN", severity = "warning",
      expected = "A Cox proportional-hazards model.", observed = "The model is not coxph.",
      details = "This diagnostic is specific to Cox models."))
  }
  z <- survival::cox.zph(info$model)
  tab <- z$table
  pcol <- grep("p", colnames(tab), ignore.case = TRUE, value = TRUE)
  p <- tab[, pcol[[1L]]]
  names(p) <- rownames(tab)
  term_p <- p[setdiff(names(p), "GLOBAL")]
  global <- if ("GLOBAL" %in% names(p)) p[["GLOBAL"]] else NA_real_
  issue <- any(term_p < 0.05, na.rm = TRUE) || (!is.na(global) && global < 0.05)
  min_term <- if (length(term_p)) min(term_p, na.rm = TRUE) else NA_real_
  .new_test(
    "cox_ph_assumption", "Proportional-hazards diagnostic", if (issue) "WARN" else "PASS",
    severity = "warning",
    expected = "No strong Schoenfeld-residual evidence against proportional hazards.",
    observed = paste0("minimum term p-value = ", .format_num(min_term),
                      "; global p-value = ", .format_num(global)),
    details = if (issue) "At least one proportional-hazards diagnostic is below 0.05; inspect time-varying effects." else "No strong proportional-hazards violation was detected by cox.zph()."
  )
}

---
title: "Introduction to statwitness"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to statwitness}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

`statwitness` creates model-aware validation certificates across computational integrity, numerical stability, design adequacy, assumption screening, and influence stability. The package changes inputs in ways that should have known consequences, refits the model, and compares the observed response with the registered expectation.

## Linear model

```{r}
library(statwitness)
dat <- transform(mtcars, transmission = factor(am))
audit <- statwitness(mpg ~ transmission + wt + hp, data = dat, focus = "transmission")
audit
```

## Mixed-effects model

The following example runs only when `lme4` is installed.

```{r, eval = requireNamespace("lme4", quietly = TRUE)}
dat <- lme4::sleepstudy
dat$criterion <- factor(rep(c("A", "B"), length.out = nrow(dat)))
dat$model <- factor(rep(c("M1", "M2"), each = 2, length.out = nrow(dat)))
dat$rater <- dat$Subject

audit <- statwitness(
  Reaction ~ criterion * model + (1 | rater),
  data = dat,
  focus = "criterion:model",
  exclude = "optimizer_agreement",
  audit_level = "core"
)
audit
```

The interaction is treated as an omnibus term. Standard mixed-model audits also inspect fixed-design conditioning, convergence, singularity, random-effect boundaries, alternative-optimizer agreement, possible omitted within-rater random slopes, residual behavior, and feasible leave-one-rater-out influence.

## Repeated-measures ANOVA

Use `statwitness_repeated()` with long-format data. It requires `afex` and applies the same audit domains with repeated-design-specific checks.

```{r, eval = FALSE}
statwitness_repeated(
  data = dataset,
  outcome = "score",
  id = "rater",
  within = c("criterion", "model"),
  focus = "criterion:model",
  audit_level = "standard"
)
```

## Scope

A passing certificate validates behavior under registered computational tests. It does not prove that the design, estimand, causal interpretation, or data-generating assumptions are scientifically correct.

\name{statwitness_repeated}
\alias{statwitness_repeated}
\title{Audit a Repeated-Measures ANOVA}
\usage{
statwitness_repeated(
  data,
  outcome,
  id,
  within,
  between = NULL,
  focus = NULL,
  type = 3,
  correction = "GG",
  tolerance = 1e-07,
  seed = 20260712,
  audit_level = c("standard", "core", "thorough")
)
}
\arguments{
\item{data}{A long-format data frame.}
\item{outcome}{Name of the numeric outcome column.}
\item{id}{Name of the participant identifier column.}
\item{within}{Character vector of within-subject factor names.}
\item{between}{Optional between-subject factor names.}
\item{focus}{Optional omnibus effect name.}
\item{type}{Sums-of-squares type passed to \code{afex::aov_ez()}.}
\item{correction}{Sphericity correction, usually \code{"GG"}.}
\item{tolerance}{Relative numerical tolerance.}
\item{seed}{Seed for row-order testing.}
\item{audit_level}{Audit breadth: \code{"core"}, \code{"standard"}, or \code{"thorough"}.}
}
\value{An object of class \code{statwitness_audit}.}
\description{
Fits a repeated-measures ANOVA using \code{afex::aov_ez()} and checks design
integrity, sphericity handling, row-order and participant-label invariance,
outcome transformations, residual screening, and feasible participant-deletion
influence checks.
}

\name{witness_registry}
\alias{witness_registry}
\title{List Registered Behavioral Audit Tests}
\usage{witness_registry()}
\value{A data frame describing registered tests, supported model families, and purposes.}
\description{Returns the built-in statwitness audit registry.}

\name{statwitness-package}
\alias{statwitness-package}
\docType{package}
\title{Model-Aware Validation and Audit Certificates for Statistical Analyses}
\description{
Model-aware computational validation for regression, ANOVA, repeated-measures,
mixed-effects, and survival workflows.
}
\details{
The package applies registered transformations with known expected behavior.
Certificates do not establish causal validity or scientific appropriateness.
}

\name{audit_plan}
\alias{audit_plan}
\title{Preview an Automatic Audit Plan}
\usage{
audit_plan(
  formula,
  data,
  focus = NULL,
  family = NULL,
  weights = NULL,
  method = "auto",
  engine = NULL,
  control = NULL,
  audit_level = c("standard", "core", "thorough")
)
}
\arguments{
\item{formula}{A model formula.}
\item{data}{A data frame.}
\item{focus}{An optional coefficient, variable, or omnibus term.}
\item{family}{An optional model family.}
\item{weights}{Optional analysis weights.}
\item{method}{Requested fitting method.}
\item{engine}{Optional custom fitting function.}
\item{control}{Optional model-specific control object.}
\item{audit_level}{Audit breadth used to construct the plan.}
}
\value{A data frame of selected and skipped tests with reasons.}
\description{Fits the requested model and reports which registered audits are applicable.}

\name{statwitness}
\alias{statwitness}
\title{Generate a Behavioral Validation Certificate}
\usage{
statwitness(
  formula,
  data,
  focus = NULL,
  family = NULL,
  weights = NULL,
  method = "auto",
  engine = NULL,
  control = NULL,
  conf_level = 0.95,
  tolerance = 1e-07,
  seed = 20260712,
  include = NULL,
  exclude = NULL,
  audit_level = c("standard", "core", "thorough")
)
}
\arguments{
\item{formula}{A model formula. Random-effect terms using \code{|} and survival responses using \code{Surv()} are supported when their suggested packages are installed.}
\item{data}{A data frame containing all analysis variables.}
\item{focus}{An optional coefficient, predictor, or omnibus term such as \code{"criterion:model"}.}
\item{family}{An optional GLM or GLMM family.}
\item{weights}{Optional analysis weights or the name of a weight column.}
\item{method}{One of \code{"auto"}, \code{"lm"}, \code{"aov"}, \code{"glm"}, \code{"lmer"}, \code{"glmer"}, \code{"glmmTMB"}, \code{"coxph"}, or \code{"survreg"}.}
\item{engine}{An optional custom fitting function.}
\item{control}{An optional model-specific control object.}
\item{conf_level}{Confidence level for coefficient intervals.}
\item{tolerance}{Relative numerical tolerance.}
\item{seed}{Seed for deterministic perturbations.}
\item{include}{Optional test identifiers to retain.}
\item{exclude}{Optional test identifiers to omit.}
\item{audit_level}{Audit breadth: \code{"core"}, \code{"standard"}, or \code{"thorough"}.}
}
\value{An object of class \code{statwitness_audit}.}
\description{
Fits a supported model, chooses mathematically applicable behavioral tests,
applies controlled transformations, and checks whether the analysis responds
as expected.
}
\examples{
dat <- transform(mtcars, transmission = factor(am))
audit <- statwitness(mpg ~ transmission + wt, data = dat, focus = "transmission")
print(audit)
}
